@@ -23,7 +23,7 @@
 struct derived_probe;
 struct match_node;
 
-struct symresolution_info: public traversing_visitor 
+struct symresolution_info: public traversing_visitor
 {
 protected:
   systemtap_session& session;
@@ -132,6 +132,10 @@ struct derived_probe: public probe
   // From within unparser::emit_probe, initialized any extra variables
   // in this probe's context locals.
 
+  virtual void emit_probe_local_init (translator_output*) {}
+  // From within unparser::emit_probe, emit any extra processing block
+  // for this probe.
+
 public:
   static void emit_common_header (translator_output* o);
   // from c_unparser::emit_common_header
@@ -181,7 +185,7 @@ typedef std::map<std::string, literal*> literal_map_t;
 struct derived_probe_builder
 {
   virtual void build(systemtap_session & sess,
-		     probe* base, 
+		     probe* base,
 		     probe_point* location,
 		     literal_map_t const & parameters,
 		     std::vector<derived_probe*> & finished_results) = 0;
@@ -256,3 +260,5 @@ struct unparser;
 
 
 #endif // ELABORATE_H
+
+/* vim: set sw=2 ts=8 cino=>4,n-2,{2,^-2,t0,(0,u0,w1,M1 : */
