@@ -1,5 +1,5 @@
 // systemtap translator/driver
-// Copyright (C) 2005-2008 Red Hat Inc.
+// Copyright (C) 2005-2009 Red Hat Inc.
 // Copyright (C) 2005 IBM Corp.
 // Copyright (C) 2006 Intel Corporation.
 //
@@ -51,7 +51,7 @@ version ()
     << "SystemTap translator/driver "
     << "(version " << VERSION << "/" << dwfl_version (NULL)
     << " " << GIT_MESSAGE << ")" << endl
-    << "Copyright (C) 2005-2008 Red Hat, Inc. and others" << endl
+    << "Copyright (C) 2005-2009 Red Hat, Inc. and others" << endl
     << "This is free software; see the source for copying conditions." << endl;
 }
 
@@ -69,24 +69,29 @@ usage (systemtap_session& s, int exitcode)
     << endl
     << "   or: stap [options] -l PROBE     List matching probes."
     << endl
+    << "   or: stap [options] -L PROBE     List matching probes and local variables."
+    << endl
     << endl
     << "Options:" << endl
     << "   --         end of translator options, script options follow" << endl
-    << "   -v         increase verbosity [" << s.verbose << "]" << endl
     << "   -h         show help" << endl
     << "   -V         show version" << endl
+    << "   -p NUM     stop after pass NUM 1-5, instead of " << s.last_pass << endl
+    << "              (parse, elaborate, translate, compile, run)" << endl
+    << "   -v         add verbosity to all passes" << endl
+    << "   --vp {N}+  add per-pass verbosity [";
+  for (unsigned i=0; i<5; i++)
+    clog << (s.perpass_verbose[i] <= 9 ? s.perpass_verbose[i] : 9);
+  clog 
+    << "]" << endl
     << "   -k         keep temporary directory" << endl
     << "   -u         unoptimized translation" << (s.unoptimized ? " [set]" : "") << endl
     << "   -w         suppress warnings" << (s.suppress_warnings ? " [set]" : "") << endl
     << "   -g         guru mode" << (s.guru_mode ? " [set]" : "") << endl
-    << "   -P         prologue-searching for function probes" 
+    << "   -P         prologue-searching for function probes"
     << (s.prologue_searching ? " [set]" : "") << endl
     << "   -b         bulk (percpu file) mode" << (s.bulk_mode ? " [set]" : "") << endl
-    << "   -s NUM     buffer size in megabytes, instead of "
-    << s.buffer_size << endl
-    << "   -p NUM     stop after pass NUM 1-5, instead of "
-    << s.last_pass << endl
-    << "              (parse, elaborate, translate, compile, run)" << endl
+    << "   -s NUM     buffer size in megabytes, instead of " << s.buffer_size << endl
     << "   -I DIR     look in DIR for additional .stp script files";
   if (s.include_path.size() == 0)
     clog << endl;
@@ -97,29 +102,40 @@ usage (systemtap_session& s, int exitcode)
   clog
     << "   -D NM=VAL  emit macro definition into generated C code" << endl
     << "   -R DIR     look in DIR for runtime, instead of" << endl
-    <<      "              " << s.runtime_path << endl
-    << "   -r RELEASE cross-compile to kernel RELEASE, instead of "
-    << s.kernel_release << endl
-    << "   -m MODULE  set probe module name, instead of "
-    << s.module_name << endl
-    << "   -o FILE    send output to file, instead of stdout" << endl
-    << "   -c CMD     start the probes, run CMD, and exit when it finishes"
-    << endl
+    << "              " << s.runtime_path << endl
+    << "   -r DIR     cross-compile to kernel with given build tree; or else" << endl
+    << "   -r RELEASE cross-compile to kernel /lib/modules/RELEASE/build, instead of" << endl
+    << "              " << s.kernel_build_tree << endl
+    << "   -m MODULE  set probe module name, instead of " << endl
+    << "              " << s.module_name << endl
+    << "   -o FILE    send script output to file, instead of stdout" << endl
+    << "   -c CMD     start the probes, run CMD, and exit when it finishes" << endl
     << "   -x PID     sets target() to PID" << endl
+    << "   -F         load module and start probes, then detach" << endl
+    << "   -d OBJECT  add unwind/symbol data for OBJECT file";
+  if (s.unwindsym_modules.size() == 0)
+    clog << endl;
+  else
+    clog << ", in addition to" << endl;
+  {
+    vector<string> syms (s.unwindsym_modules.begin(), s.unwindsym_modules.end());
+    for (unsigned i=0; i<syms.size(); i++)
+      clog << "              " << syms[i] << endl;
+  }
+  clog
     << "   -t         collect probe timing information" << endl
 #ifdef HAVE_LIBSQLITE3
     << "   -q         generate information on tapset coverage" << endl
 #endif /* HAVE_LIBSQLITE3 */
+#if 0 /* PR6864: disable temporarily; should merge with -d somehow */
     << "   --kelf     make do with symbol table from vmlinux" << endl
     << "   --kmap[=FILE]" << endl
     << "              make do with symbol table from nm listing" << endl
-    << "   --ignore-vmlinux" << endl
-    << "              for testing, pretend vmlinux can't be found" << endl
-    << "   --ignore-dwarf" << endl
-    << "              for testing, pretend vmlinux and modules lack debug info"
+#endif
+  // Formerly present --ignore-{vmlinux,dwarf} options are for testsuite use
+  // only, and don't belong in the eyesight of a plain user.
     << endl
     ;
-  // -d: dump safety-related external references
 
   exit (exitcode);
 }
@@ -135,6 +151,8 @@ printscript(systemtap_session& s, ostream& o)
 
       for (unsigned i=0; i<s.probes.size(); i++)
         {
+          if (pending_interrupts) return;
+
           derived_probe* p = s.probes[i];
           // NB: p->basest() is not so interesting;
           // p->almost_basest() doesn't quite work, so ...
@@ -154,7 +172,7 @@ printscript(systemtap_session& s, ostream& o)
             {
               assert (a->alias_names.size() >= 1);
               a->alias_names[0]->print(tmps); // XXX: [0] is arbitrary; perhaps print all
-            }            
+            }
           else
             {
               assert (second->locations.size() >= 1);
@@ -167,7 +185,16 @@ printscript(systemtap_session& s, ostream& o)
           // print the alias head name once.
           if (seen.find (pp) == seen.end())
             {
-              o << pp << endl;
+              o << pp;
+              // Print the locals for -L mode only
+              if (s.unoptimized)
+                for (unsigned j=0; j<p->locals.size(); j++)
+                  {
+                    o << " ";
+                    vardecl* v = p->locals[j];
+                    v->printsig (o);
+                  }
+              o << endl;
               seen.insert (pp);
             }
         }
@@ -178,15 +205,17 @@ printscript(systemtap_session& s, ostream& o)
         o << "# global embedded code" << endl;
       for (unsigned i=0; i<s.embeds.size(); i++)
         {
+          if (pending_interrupts) return;
           embeddedcode* ec = s.embeds[i];
           ec->print (o);
           o << endl;
         }
-      
+
       if (s.globals.size() > 0)
         o << "# globals" << endl;
       for (unsigned i=0; i<s.globals.size(); i++)
         {
+          if (pending_interrupts) return;
           vardecl* v = s.globals[i];
           v->printsig (o);
           if (s.verbose && v->init)
@@ -196,12 +225,13 @@ printscript(systemtap_session& s, ostream& o)
             }
           o << endl;
         }
-      
+
       if (s.functions.size() > 0)
         o << "# functions" << endl;
-      for (unsigned i=0; i<s.functions.size(); i++)
+      for (map<string,functiondecl*>::iterator it = s.functions.begin(); it != s.functions.end(); it++)
         {
-          functiondecl* f = s.functions[i];
+          if (pending_interrupts) return;
+          functiondecl* f = it->second;
           f->printsig (o);
           o << endl;
           if (f->locals.size() > 0)
@@ -219,11 +249,12 @@ printscript(systemtap_session& s, ostream& o)
               o << endl;
             }
         }
-      
+
       if (s.probes.size() > 0)
         o << "# probes" << endl;
       for (unsigned i=0; i<s.probes.size(); i++)
         {
+          if (pending_interrupts) return;
           derived_probe* p = s.probes[i];
           p->printsig (o);
           o << endl;
@@ -249,10 +280,16 @@ printscript(systemtap_session& s, ostream& o)
 int pending_interrupts;
 
 extern "C"
-void handle_interrupt (int /* sig */) 
+void handle_interrupt (int sig)
 {
+  if (pending_interrupts == 0)
+    kill (0, sig); // Forward signals to child processes if any.
+
   pending_interrupts ++;
-  if (pending_interrupts > 1) // XXX: should be configurable? time-based?
+  // NB: the "2" below is intended to skip the effect of the self-induced
+  // deferred signal coming from the kill() above.
+
+  if (pending_interrupts > 2) // XXX: should be configurable? time-based?
     {
       char msg[] = "Too many interrupts received, exiting.\n";
       int rc = write (2, msg, sizeof(msg)-1);
@@ -291,7 +328,6 @@ main (int argc, char * const argv [])
   string cmdline_script; // -e PROGRAM
   string script_file; // FILE
   bool have_script = false;
-  bool release_changed = false;
   bool save_module = false;
 
   // Initialize defaults
@@ -299,8 +335,10 @@ main (int argc, char * const argv [])
   struct utsname buf;
   (void) uname (& buf);
   s.kernel_release = string (buf.release);
+  s.kernel_build_tree = "/lib/modules/" + s.kernel_release + "/build";
+
   s.architecture = string (buf.machine);
-  s.verbose = 0;
+  for (unsigned i=0; i<5; i++) s.perpass_verbose[i]=0;
   s.timing = false;
   s.guru_mode = false;
   s.bulk_mode = false;
@@ -317,6 +355,7 @@ main (int argc, char * const argv [])
   s.buffer_size = 0;
   s.last_pass = 5;
   s.module_name = "stap_" + stringify(getpid());
+  s.stapconf_name = "stapconf_" + stringify(getpid()) + ".h";
   s.output_file = ""; // -o FILE
   s.keep_tmpdir = false;
   s.cmd = "";
@@ -330,9 +369,10 @@ main (int argc, char * const argv [])
   s.consult_symtab = false;
   s.ignore_vmlinux = false;
   s.ignore_dwarf = false;
+  s.load_only = false;
 
   const char* s_p = getenv ("SYSTEMTAP_TAPSET");
-  if (s_p != NULL)  
+  if (s_p != NULL)
   {
     s.include_path.push_back (s_p);
   }
@@ -355,9 +395,10 @@ main (int argc, char * const argv [])
   if (create_dir(s.data_path.c_str()) == 1)
     {
       const char* e = strerror (errno);
-      cerr << "Warning: failed to create systemtap data directory (\""
-	   << s.data_path << "\"): " << e << endl;
-      cerr << "Disabling cache support." << endl;
+      if (! s.suppress_warnings)
+        cerr << "Warning: failed to create systemtap data directory (\""
+             << s.data_path << "\"): " << e
+             << ", disabling cache support." << endl;
       s.use_cache = false;
     }
 
@@ -367,9 +408,10 @@ main (int argc, char * const argv [])
       if (create_dir(s.cache_path.c_str()) == 1)
         {
 	  const char* e = strerror (errno);
-	  cerr << "Warning: failed to create cache directory (\""
-	       << s.cache_path << "\"): " << e << endl;
-	  cerr << "Disabling cache support." << endl;
+          if (! s.suppress_warnings)
+            cerr << "Warning: failed to create cache directory (\""
+                 << s.cache_path << "\"): " << e
+                 << ", disabling cache support." << endl;
 	  s.use_cache = false;
 	}
     }
@@ -385,15 +427,17 @@ main (int argc, char * const argv [])
 #define LONG_OPT_KMAP 2
 #define LONG_OPT_IGNORE_VMLINUX 3
 #define LONG_OPT_IGNORE_DWARF 4
+#define LONG_OPT_VERBOSE_PASS 5
       // NB: also see find_hash(), usage(), switch stmt below, stap.1 man page
       static struct option long_options[] = {
         { "kelf", 0, &long_opt, LONG_OPT_KELF },
         { "kmap", 2, &long_opt, LONG_OPT_KMAP },
         { "ignore-vmlinux", 0, &long_opt, LONG_OPT_IGNORE_VMLINUX },
         { "ignore-dwarf", 0, &long_opt, LONG_OPT_IGNORE_DWARF },
+        { "vp", 1, &long_opt, LONG_OPT_VERBOSE_PASS },
         { NULL, 0, NULL, 0 }
       };
-      int grc = getopt_long (argc, argv, "hVMvtp:I:e:o:R:r:m:kgPc:x:D:bs:uqwl:",
+      int grc = getopt_long (argc, argv, "hVMvtp:I:e:o:R:r:m:kgPc:x:D:bs:uqwl:d:L:F",
                                                           long_options, NULL);
       if (grc < 0)
         break;
@@ -408,7 +452,8 @@ main (int argc, char * const argv [])
           break;
 
         case 'v':
-	  s.verbose ++;
+          for (unsigned i=0; i<5; i++)
+            s.perpass_verbose[i] ++;
 	  break;
 
         case 't':
@@ -437,6 +482,10 @@ main (int argc, char * const argv [])
           s.include_path.push_back (string (optarg));
           break;
 
+        case 'd':
+          s.unwindsym_modules.insert (string (optarg));
+          break;
+
         case 'e':
 	  if (have_script)
 	    {
@@ -503,12 +552,36 @@ main (int argc, char * const argv [])
           break;
 
         case 'r':
-          s.kernel_release = string (optarg);
-	  release_changed = true;
+          if (optarg[0] == '/') // fully specified path
+            {
+              s.kernel_build_tree = optarg;
+              string version_file_name = s.kernel_build_tree + "/include/config/kernel.release";
+              // The file include/config/kernel.release within the
+              // build tree is used to pull out the version information
+              ifstream version_file (version_file_name.c_str());
+              if (version_file.fail ())
+                {
+                  cerr << "Missing " << version_file_name << endl;
+                  usage (s, 1);
+                }
+              else 
+                {
+                  char c;
+                  s.kernel_release = "";
+                  while (version_file.get(c) && c != '\n')
+                    s.kernel_release.push_back(c);
+                }
+            }
+          else
+            {
+              s.kernel_release = string (optarg);
+              s.kernel_build_tree = "/lib/modules/" + s.kernel_release + "/build";
+            }
           break;
 
         case 'k':
           s.keep_tmpdir = true;
+          s.use_cache = false; /* User wants to keep a usable build tree. */
           break;
 
         case 'g':
@@ -556,6 +629,9 @@ main (int argc, char * const argv [])
           usage (s, 0);
           break;
 
+        case 'L':
+          s.unoptimized = true; // This causes retention of variables for listing_mode
+
         case 'l':
 	  s.suppress_warnings = true;
           s.listing_mode = true;
@@ -570,6 +646,10 @@ main (int argc, char * const argv [])
           have_script = true;
           break;
 
+        case 'F':
+          s.load_only = true;
+	  break;
+
         case 0:
           switch (long_opt)
             {
@@ -595,6 +675,26 @@ main (int argc, char * const argv [])
 	    case LONG_OPT_IGNORE_DWARF:
 	      s.ignore_dwarf = true;
 	      break;
+	    case LONG_OPT_VERBOSE_PASS:
+              {
+                bool ok = true;
+                if (strlen(optarg) < 1 || strlen(optarg) > 5)
+                  ok = false;
+                if (ok)
+                  for (unsigned i=0; i<strlen(optarg); i++)
+                    if (isdigit (optarg[i]))
+                      s.perpass_verbose[i] += optarg[i]-'0';
+                    else
+                      ok = false;
+                
+                if (! ok)
+                  {
+                    cerr << "Invalid --vp argument: it takes 1 to 5 digits." << endl;
+                    usage (s, 1);
+                  }
+                // NB: we don't do this: s.last_pass = strlen(optarg);
+                break;
+              }
             default:
               cerr << "Internal error parsing command arguments." << endl;
               usage(s, 1);
@@ -637,12 +737,15 @@ main (int argc, char * const argv [])
         s.kernel_symtab_path = string("/boot/System.map-") + s.kernel_release;
     }
 
-  if (s.last_pass > 4 && release_changed)
-    {
-      if (s.verbose)
-        cerr << "Warning: changing last pass to 4 since cross-compiling" << endl;
-      s.last_pass = 4;
-    }
+  // Warn in case the target kernel release doesn't match the running one.
+  if (s.last_pass > 4 &&
+      (string(buf.release) != s.kernel_release ||
+       string(buf.machine) != s.architecture))
+   {
+     if(! s.suppress_warnings)
+       cerr << "WARNING: kernel release/architecture mismatch with host forces last-pass 4." << endl;
+     s.last_pass = 4;
+   }
 
   for (int i = optind; i < argc; i++)
     {
@@ -673,26 +776,15 @@ main (int argc, char * const argv [])
     }
 
   int rc = 0;
+  
+  // PASS 0: setting up
+  s.verbose = s.perpass_verbose[0];
 
-  // override PATH and LC_ALL
-  const char *path = "/bin:/sbin:/usr/bin:/usr/sbin";
-  rc = setenv("PATH", path, 1) || setenv("LC_ALL", "C", 1);
-  if (rc)
-    {
-      const char* e = strerror (errno);
-      cerr << "setenv (\"PATH=" << path << "\" + \"LC_ALL=C\"): "
-           << e << endl;
-    }
-
-  // Get rid of a few standard environment variables (which might
-  // cause us to do unintended things).
-  rc = unsetenv("IFS") || unsetenv("CDPATH") || unsetenv("ENV")
-      || unsetenv("BASH_ENV");
-  if (rc)
-    {
-      const char* e = strerror (errno);
-      cerr << "unsetenv failed: " << e << endl;
-    }
+  // For PR1477, we used to override $PATH and $LC_ALL and other stuff
+  // here.  We seem to use complete pathnames in
+  // buildrun.cxx/tapsets.cxx now, so this is not necessary.  Further,
+  // it interferes with util.cxx:find_executable(), used for $PATH
+  // resolution.
 
   s.kernel_base_release.assign(s.kernel_release, 0, s.kernel_release.find('-'));
 
@@ -711,7 +803,7 @@ main (int argc, char * const argv [])
     const char* tmpdir_env = getenv("TMPDIR");
     if (! tmpdir_env)
       tmpdir_env = "/tmp";
-    
+
     string stapdir = "/stapXXXXXX";
     string tmpdirt = tmpdir_env + stapdir;
     mode_t mask = umask(0);
@@ -734,7 +826,7 @@ main (int argc, char * const argv [])
   // directory.
   s.translated_source = string(s.tmpdir) + "/" + s.module_name + ".c";
 
-  // Set up our handler to catch routine signals, to allow clean 
+  // Set up our handler to catch routine signals, to allow clean
   // and reasonably timely exit.
   setup_signals(&handle_interrupt);
 
@@ -877,9 +969,9 @@ main (int argc, char * const argv [])
            << endl;
     }
 
-  if (rc)
+  if (rc && !s.listing_mode)
     cerr << "Pass 1: parse failed.  "
-         << "Try again with more '-v' (verbose) options."
+         << "Try again with another '--vp 1' option."
          << endl;
 
   if (rc || s.last_pass == 1 || pending_interrupts) goto cleanup;
@@ -888,6 +980,7 @@ main (int argc, char * const argv [])
   gettimeofday (&tv_before, NULL);
 
   // PASS 2: ELABORATION
+  s.verbose = s.perpass_verbose[1];
   rc = semantic_pass (s);
 
   if (s.listing_mode || (rc == 0 && s.last_pass == 2))
@@ -904,9 +997,9 @@ main (int argc, char * const argv [])
                       << TIMESPRINT
                       << endl;
 
-  if (rc)
+  if (rc && !s.listing_mode)
     cerr << "Pass 2: analysis failed.  "
-         << "Try again with more '-v' (verbose) options."
+         << "Try again with another '--vp 01' option."
          << endl;
   // Generate hash.  There isn't any point in generating the hash
   // if last_pass is 2, since we'll quit before using it.
@@ -942,7 +1035,7 @@ main (int argc, char * const argv [])
   if (rc || s.listing_mode || s.last_pass == 2 || pending_interrupts) goto cleanup;
 
   // PASS 3: TRANSLATION
-
+  s.verbose = s.perpass_verbose[2];
   times (& tms_before);
   gettimeofday (&tv_before, NULL);
 
@@ -965,12 +1058,13 @@ main (int argc, char * const argv [])
 
   if (rc)
     cerr << "Pass 3: translation failed.  "
-         << "Try again with more '-v' (verbose) options."
+         << "Try again with another '--vp 001' option."
          << endl;
 
   if (rc || s.last_pass == 3 || pending_interrupts) goto cleanup;
 
   // PASS 4: COMPILATION
+  s.verbose = s.perpass_verbose[3];
   times (& tms_before);
   gettimeofday (&tv_before, NULL);
   rc = compile_pass (s);
@@ -992,14 +1086,19 @@ main (int argc, char * const argv [])
 
   if (rc)
     cerr << "Pass 4: compilation failed.  "
-         << "Try again with more '-v' (verbose) options."
+         << "Try again with another '--vp 0001' option."
          << endl;
   else
     {
-      // Update cache.
+      // Update cache. Cache cleaning is kicked off at the end of this function.
       if (s.use_cache)
-	add_to_cache(s);
+        add_to_cache(s);
 
+      // We may need to save the module in $CWD if the cache was
+      // inaccessible for some reason.
+      if (! s.use_cache && s.last_pass == 4)
+        save_module = true;
+      
       // Copy module to the current directory.
       if (save_module && !pending_interrupts)
         {
@@ -1020,6 +1119,7 @@ main (int argc, char * const argv [])
 
   // PASS 5: RUN
 pass_5:
+  s.verbose = s.perpass_verbose[4];
   times (& tms_before);
   gettimeofday (&tv_before, NULL);
   // NB: this message is a judgement call.  The other passes don't emit
@@ -1035,11 +1135,12 @@ pass_5:
 
   if (rc)
     cerr << "Pass 5: run failed.  "
-         << "Try again with more '-v' (verbose) options."
+         << "Try again with another '--vp 00001' option."
          << endl;
 
   // if (rc) goto cleanup;
 
+  // PASS 6: cleaning up
  cleanup:
 
   // update the database information
@@ -1075,3 +1176,5 @@ pass_5:
 
   return (rc||pending_interrupts) ? EXIT_FAILURE : EXIT_SUCCESS;
 }
+
+/* vim: set sw=2 ts=8 cino=>4,n-2,{2,^-2,t0,(0,u0,w1,M1 : */
