@@ -15,6 +15,7 @@
 #include "util.h"
 
 #include <iostream>
+
 #include <fstream>
 #include <cctype>
 #include <cstdlib>
@@ -24,6 +25,11 @@
 #include <sstream>
 #include <cstring>
 #include <cctype>
+#include <iterator>
+
+extern "C" {
+#include <fnmatch.h>
+}
 
 using namespace std;
 
@@ -85,7 +91,7 @@ tt2str(token_type tt)
 ostream&
 operator << (ostream& o, const source_loc& loc)
 {
-  o << loc.file << ":" 
+  o << loc.file << ":"
     << loc.line << ":"
     << loc.column;
 
@@ -108,27 +114,32 @@ operator << (ostream& o, const token& t)
       o << "'";
     }
 
-  o << " at " 
+  o << " at "
     << t.location;
 
   return o;
 }
 
 
-void 
+void
 parser::print_error  (const parse_error &pe)
 {
+  string align_parse_error ("     ");
   cerr << "parse error: " << pe.what () << endl;
 
   if (pe.tok)
     {
       cerr << "\tat: " << *pe.tok << endl;
+      session.print_error_source (cerr, align_parse_error, pe.tok);
     }
   else
     {
       const token* t = last_t;
       if (t)
-        cerr << "\tsaw: " << *t << endl;
+	{
+	  cerr << "\tsaw: " << *t << endl;
+	  session.print_error_source (cerr, align_parse_error, t);
+	}
       else
         cerr << "\tsaw: " << input_name << " EOF" << endl;
     }
@@ -140,13 +151,34 @@ parser::print_error  (const parse_error &pe)
 }
 
 
-const token* 
+const token*
 parser::last ()
 {
   return last_t;
 }
 
 
+
+template <typename OPERAND>
+bool eval_comparison (const OPERAND& lhs, const token* op, const OPERAND& rhs)
+{
+  if (op->type == tok_operator && op->content == "<=")
+    { return lhs <= rhs; }
+  else if (op->type == tok_operator && op->content == ">=")
+    { return lhs >= rhs; }
+  else if (op->type == tok_operator && op->content == "<")
+    { return lhs < rhs; }
+  else if (op->type == tok_operator && op->content == ">")
+    { return lhs > rhs; }
+  else if (op->type == tok_operator && op->content == "==")
+    { return lhs == rhs; }
+  else if (op->type == tok_operator && op->content == "!=")
+    { return lhs != rhs; }
+  else
+    throw parse_error ("expected comparison operator", op);
+}
+
+
 // Here, we perform on-the-fly preprocessing.
 // The basic form is %( CONDITION %? THEN-TOKENS %: ELSE-TOKENS %)
 // where CONDITION is: kernel_v[r] COMPARISON-OP "version-string"
@@ -156,7 +188,7 @@ parser::last ()
 // The %: ELSE-TOKENS part is optional.
 //
 // e.g. %( kernel_v > "2.5" %? "foo" %: "baz" %)
-// e.g. %( arch != "i686" %? "foo" %: "baz" %)
+// e.g. %( arch != "i?86" %? "foo" %: "baz" %)
 //
 // Up to an entire %( ... %) expression is processed by a single call
 // to this function.  Tokens included by any nested conditions are
@@ -170,13 +202,19 @@ bool eval_pp_conditional (systemtap_session& s,
     {
       string target_kernel_vr = s.kernel_release;
       string target_kernel_v = s.kernel_base_release;
-      
+
       if (! (r->type == tok_string))
         throw parse_error ("expected string literal", r);
-      string query_kernel_vr = r->content;
-      
+
+      string target = (l->content == "kernel_vr" ?
+                       target_kernel_vr.c_str() :
+                       target_kernel_v.c_str());
+      string query = r->content;
+      bool rhs_wildcard = (strpbrk (query.c_str(), "*?[") != 0);
+
       // collect acceptable strverscmp results.
       int rvc_ok1, rvc_ok2;
+      bool wc_ok = false;
       if (op->type == tok_operator && op->content == "<=")
         { rvc_ok1 = -1; rvc_ok2 = 0; }
       else if (op->type == tok_operator && op->content == ">=")
@@ -186,21 +224,30 @@ bool eval_pp_conditional (systemtap_session& s,
       else if (op->type == tok_operator && op->content == ">")
         { rvc_ok1 = 1; rvc_ok2 = 1; }
       else if (op->type == tok_operator && op->content == "==")
-        { rvc_ok1 = 0; rvc_ok2 = 0; }
+        { rvc_ok1 = 0; rvc_ok2 = 0; wc_ok = true; }
       else if (op->type == tok_operator && op->content == "!=")
-        { rvc_ok1 = -1; rvc_ok2 = 1; }
+        { rvc_ok1 = -1; rvc_ok2 = 1; wc_ok = true; }
       else
         throw parse_error ("expected comparison operator", op);
-      
-      int rvc_result = strverscmp ((l->content == "kernel_vr" ? 
-                                    target_kernel_vr.c_str() :
-                                    target_kernel_v.c_str()),
-                                   query_kernel_vr.c_str());
-      // normalize rvc_result
-      if (rvc_result < 0) rvc_result = -1;
-      if (rvc_result > 0) rvc_result = 1;
-      
-      return (rvc_result == rvc_ok1 || rvc_result == rvc_ok2);
+
+      if ((!wc_ok) && rhs_wildcard)
+        throw parse_error ("wildcard not allowed with order comparison operators", op);
+
+      if (rhs_wildcard)
+        {
+          int rvc_result = fnmatch (query.c_str(), target.c_str(),
+                                    FNM_NOESCAPE); // spooky
+          bool badness = (rvc_result == 0) ^ (op->content == "==");
+          return !badness;
+        }
+      else
+        {
+          int rvc_result = strverscmp (target.c_str(), query.c_str());
+          // normalize rvc_result
+          if (rvc_result < 0) rvc_result = -1;
+          if (rvc_result > 0) rvc_result = 1;
+          return (rvc_result == rvc_ok1 || rvc_result == rvc_ok2);
+        }
     }
   else if (l->type == tok_identifier && l->content == "arch")
     {
@@ -208,44 +255,34 @@ bool eval_pp_conditional (systemtap_session& s,
       if (! (r->type == tok_string))
         throw parse_error ("expected string literal", r);
       string query_architecture = r->content;
-      
+
+      int nomatch = fnmatch (query_architecture.c_str(),
+                             target_architecture.c_str(),
+                             FNM_NOESCAPE); // still spooky
+
       bool result;
       if (op->type == tok_operator && op->content == "==")
-        result = target_architecture == query_architecture;
+        result = !nomatch;
       else if (op->type == tok_operator && op->content == "!=")
-        result = target_architecture != query_architecture;
+        result = nomatch;
       else
         throw parse_error ("expected '==' or '!='", op);
-      
+
       return result;
-    }  
-  else if ((l->type == tok_string && r->type == tok_string)
-	   || (l->type == tok_number && r->type == tok_number))
+    }
+  else if (l->type == tok_string && r->type == tok_string)
     {
-      // collect acceptable strverscmp results.
-      int rvc_ok1, rvc_ok2;
-      if (op->type == tok_operator && op->content == "<=")
-        { rvc_ok1 = -1; rvc_ok2 = 0; }
-      else if (op->type == tok_operator && op->content == ">=")
-        { rvc_ok1 = 1; rvc_ok2 = 0; }
-      else if (op->type == tok_operator && op->content == "<")
-        { rvc_ok1 = -1; rvc_ok2 = -1; }
-      else if (op->type == tok_operator && op->content == ">")
-        { rvc_ok1 = 1; rvc_ok2 = 1; }
-      else if (op->type == tok_operator && op->content == "==")
-        { rvc_ok1 = 0; rvc_ok2 = 0; }
-      else if (op->type == tok_operator && op->content == "!=")
-        { rvc_ok1 = -1; rvc_ok2 = 1; }
-      else
-        throw parse_error ("expected comparison operator", op);
-
-      int rvc_result = l->content.compare(r->content);
-
-      // normalize rvc_result
-      if (rvc_result < 0) rvc_result = -1;
-      if (rvc_result > 0) rvc_result = 1;
-
-      return (rvc_result == rvc_ok1 || rvc_result == rvc_ok2);
+      string lhs = l->content;
+      string rhs = r->content;
+      return eval_comparison (lhs, op, rhs);
+      // NB: no wildcarding option here
+    }
+  else if (l->type == tok_number && r->type == tok_number)
+    {
+      int64_t lhs = lex_cast<int64_t>(l->content);
+      int64_t rhs = lex_cast<int64_t>(r->content);
+      return eval_comparison (lhs, op, rhs);
+      // NB: no wildcarding option here
     }
   else if (l->type == tok_string && r->type == tok_number
 	    && op->type == tok_operator)
@@ -253,17 +290,18 @@ bool eval_pp_conditional (systemtap_session& s,
   else if (l->type == tok_number && r->type == tok_string
 	    && op->type == tok_operator)
     throw parse_error ("expected number literal as right value", r);
+
   // XXX: support other forms?  "CONFIG_SMP" ?
+
   else
     throw parse_error ("expected 'arch' or 'kernel_v' or 'kernel_vr'\n"
 		       "             or comparison between strings or integers", l);
 }
 
 
-// expand_args is used to know if we must expand $x and @x identifiers.
 // Only tokens corresponding to the TRUE statement must be expanded
 const token*
-parser::scan_pp (bool expand_args)
+parser::scan_pp (bool wildcard)
 {
   while (true)
     {
@@ -274,87 +312,118 @@ parser::scan_pp (bool expand_args)
           return t;
         }
 
-      const token* t = input.scan (expand_args); // NB: not recursive!
+      const token* t = input.scan (wildcard); // NB: not recursive!
       if (t == 0) // EOF
         return t;
-      
+
       if (! (t->type == tok_operator && t->content == "%(")) // ordinary token
         return t;
 
       // We have a %( - it's time to throw a preprocessing party!
 
       const token *l, *op, *r;
-      l = input.scan (expand_args); // NB: not recursive, though perhaps could be
-      op = input.scan (expand_args);
-      r = input.scan (expand_args);
+      l = input.scan (false); // NB: not recursive, though perhaps could be
+      op = input.scan (false);
+      r = input.scan (false);
       if (l == 0 || op == 0 || r == 0)
         throw parse_error ("incomplete condition after '%('", t);
       // NB: consider generalizing to consume all tokens until %?, and
       // passing that as a vector to an evaluator.
 
       // Do not evaluate the condition if we haven't expanded everything.
-      // This may occured when having several recursive conditionals.
-      bool result = expand_args && eval_pp_conditional (session, l, op, r);
+      // This may occur when having several recursive conditionals.
+      bool result = eval_pp_conditional (session, l, op, r);
       delete l;
       delete op;
       delete r;
-      
+
+      /*
+      clog << "PP eval (" << *t << ") == " << result << endl;
+      */
+
       const token *m = input.scan (); // NB: not recursive
       if (! (m && m->type == tok_operator && m->content == "%?"))
         throw parse_error ("expected '%?' marker for conditional", t);
       delete m; // "%?"
 
       vector<const token*> my_enqueued_pp;
-      bool have_token = false;
-      
+
+      int nesting = 0;
       while (true) // consume THEN tokens
         {
-          m = scan_pp (result); // NB: recursive
-          if (m == 0)
-            throw parse_error (have_token ?
-                               "incomplete conditional - missing %: or %)" :
-                               "missing THEN tokens for conditional",
-                               t);
+          try
+            {
+              m = result ? scan_pp (wildcard) : input.scan (wildcard);
+            }
+          catch (const parse_error &e)
+            {
+              if (result) throw e; // propagate errors if THEN branch taken
+              continue;
+            }
 
-	  have_token = true;
-          if (m->type == tok_operator && (m->content == "%:" || // ELSE
-                                          m->content == "%)")) // END
+          if (m && m->type == tok_operator && m->content == "%(") // nested %(
+            nesting ++;
+          if (nesting == 0 && m && (m->type == tok_operator && (m->content == "%:" || // ELSE
+                                                                m->content == "%)"))) // END
             break;
-          // enqueue token
-          if (result) 
+          if (nesting && m && m->type == tok_operator && m->content == "%)") // nested %)
+            nesting --;
+
+          if (!m)
+            throw parse_error ("incomplete conditional - missing '%:' or '%)'", t);
+          if (result)
             my_enqueued_pp.push_back (m);
-          else
-            delete m; // unused token
-          // continue
+          if (!result)
+            delete m; // do nothing, just dispose of unkept THEN token
+
+          continue;
         }
-      
-      have_token = false;
+
       if (m && m->type == tok_operator && m->content == "%:") // ELSE
         {
           delete m; // "%:"
+          int nesting = 0;
           while (true)
             {
-              m = scan_pp (expand_args && !result); // NB: recursive
-              if (m == 0)
-		  throw parse_error (have_token ?
-                                     "incomplete conditional - missing %)" :
-                                     "missing ELSE tokens for conditional",
-                                     t);
+              try
+                {
+                  m = result ? input.scan (wildcard) : scan_pp (wildcard);
+                }
+              catch (const parse_error& e)
+                {
+                  if (!result) throw e; // propagate errors if ELSE branch taken
+                  continue;
+                }
 
-	      have_token = true;
-              if (m->type == tok_operator && m->content == "%)") // END
+              if (m && m->type == tok_operator && m->content == "%(") // nested %(
+                nesting ++;
+              if (nesting == 0 && m && m->type == tok_operator && m->content == "%)") // END
                 break;
-              // enqueue token
-              if (! result) 
+              if (nesting && m && m->type == tok_operator && m->content == "%)") // nested %)
+                nesting --;
+
+              if (!m)
+                throw parse_error ("incomplete conditional - missing %)", t);
+              if (!result)
                 my_enqueued_pp.push_back (m);
-              else
-                delete m; // unused token
-              // continue
+              if (result)
+                delete m; // do nothing, just dispose of unkept ELSE token
+
+              continue;
             }
         }
+
+      /*
+      clog << "PP eval (" << *t << ") == " << result << " tokens: " << endl;
+      for (unsigned k=0; k<my_enqueued_pp.size(); k++)
+        clog << * my_enqueued_pp[k] << endl;
+      clog << endl;
+      */
+
       delete t; // "%("
       delete m; // "%)"
 
+
       // NB: we transcribe the retained tokens here, and not inside
       // the THEN/ELSE while loops.  If it were done there, each loop
       // would become infinite (each iteration consuming an ordinary
@@ -372,10 +441,10 @@ parser::scan_pp (bool expand_args)
 
 
 const token*
-parser::next ()
+parser::next (bool wildcard)
 {
   if (! next_t)
-    next_t = scan_pp ();
+    next_t = scan_pp (wildcard);
   if (! next_t)
     throw parse_error ("unexpected end-of-file");
 
@@ -387,10 +456,10 @@ parser::next ()
 
 
 const token*
-parser::peek ()
+parser::peek (bool wildcard)
 {
   if (! next_t)
-    next_t = scan_pp ();
+    next_t = scan_pp (wildcard);
 
   // don't advance by zeroing next_t
   last_t = next_t;
@@ -405,7 +474,7 @@ tok_is(token const * t, token_type tt, string const & expected)
 }
 
 
-const token* 
+const token*
 parser::expect_known (token_type tt, string const & expected)
 {
   const token *t = next();
@@ -415,7 +484,7 @@ parser::expect_known (token_type tt, string const & expected)
 }
 
 
-const token* 
+const token*
 parser::expect_unknown (token_type tt, string & target)
 {
   const token *t = next();
@@ -426,7 +495,7 @@ parser::expect_unknown (token_type tt, string & target)
 }
 
 
-const token* 
+const token*
 parser::expect_unknown2 (token_type tt1, token_type tt2, string & target)
 {
   const token *t = next();
@@ -437,20 +506,20 @@ parser::expect_unknown2 (token_type tt1, token_type tt2, string & target)
 }
 
 
-const token* 
+const token*
 parser::expect_op (std::string const & expected)
 {
   return expect_known (tok_operator, expected);
 }
 
 
-const token* 
+const token*
 parser::expect_kw (std::string const & expected)
 {
   return expect_known (tok_identifier, expected);
 }
 
-const token* 
+const token*
 parser::expect_number (int64_t & value)
 {
   bool neg = false;
@@ -476,8 +545,8 @@ parser::expect_number (int64_t & value)
       || (neg && (unsigned long long) value > 9223372036854775808ULL)
       || (unsigned long long) value > 18446744073709551615ULL
       || value < -9223372036854775807LL-1)
-    throw parse_error ("number invalid or out of range"); 
-  
+    throw parse_error ("number invalid or out of range");
+
   if (neg)
     value = -value;
 
@@ -485,28 +554,28 @@ parser::expect_number (int64_t & value)
 }
 
 
-const token* 
+const token*
 parser::expect_ident (std::string & target)
 {
   return expect_unknown (tok_identifier, target);
 }
 
 
-const token* 
+const token*
 parser::expect_ident_or_keyword (std::string & target)
 {
   return expect_unknown2 (tok_identifier, tok_keyword, target);
 }
 
 
-bool 
+bool
 parser::peek_op (std::string const & op)
 {
   return tok_is (peek(), tok_operator, op);
 }
 
 
-bool 
+bool
 parser::peek_kw (std::string const & kw)
 {
   return tok_is (peek(), tok_identifier, kw);
@@ -515,28 +584,43 @@ parser::peek_kw (std::string const & kw)
 
 
 lexer::lexer (istream& i, const string& in, systemtap_session& s):
-  input (i), input_name (in), cursor_suspend_count(0), 
-  cursor_line (1), cursor_column (1), session(s)
-{ }
+  input (i), input_name (in), input_contents (""),
+  input_pointer (0), cursor_suspend_count(0),
+  cursor_line (1), cursor_column (1), session(s),
+  current_file (0)
+{
+  char c;
+  while(input.get(c))
+    input_contents.push_back(c);
+}
 
+std::string
+lexer::get_input_contents ()
+{
+  return input_contents;
+}
+
+void
+lexer::set_current_file (stapfile* f)
+{
+  current_file = f;
+}
 
 int
 lexer::input_peek (unsigned n)
 {
-  while (lookahead.size() <= n)
-    {
-      int c = input.get ();
-      lookahead.push_back (input ? c : -1);
-    }
-  return lookahead[n];
+  if (input_contents.size() > (input_pointer + n))
+    return (int)(unsigned char)input_contents[input_pointer+n];
+  else
+    return -1;
 }
 
 
-int 
+int
 lexer::input_get ()
 {
   int c = input_peek (0);
-  lookahead.erase (lookahead.begin ());
+  input_pointer ++;
 
   if (c < 0) return c; // EOF
 
@@ -556,6 +640,7 @@ lexer::input_get ()
         cursor_column ++;
     }
 
+  // clog << "[" << (char)c << "]";
   return c;
 }
 
@@ -563,21 +648,19 @@ lexer::input_get ()
 void
 lexer::input_put (const string& chars)
 {
-  // clog << "[put:" << chars << "]";
-  for (int i=chars.size()-1; i>=0; i--)
-    {
-      int c = chars[i];
-      lookahead.insert (lookahead.begin(), c);
-      cursor_suspend_count ++;
-    }
+  // clog << "[put:" << chars << " @" << input_pointer << "]";
+  input_contents.insert (input_contents.begin() + input_pointer, chars.begin(), chars.end());
+  cursor_suspend_count += chars.size();
 }
 
 
 token*
-lexer::scan (bool expand_args)
+lexer::scan (bool wildcard)
 {
   token* n = new token;
   n->location.file = input_name;
+  if (current_file)
+    n->location.stap_file = current_file;
 
   unsigned semiskipped_p = 0;
 
@@ -609,8 +692,7 @@ lexer::scan (bool expand_args)
   // characters; @1..@999 are quoted/escaped as strings.
   // $# and @# expand to the number of arguments, similarly
   // raw or quoted.
-  if (expand_args &&
-      (c == '$' || c == '@') &&
+  if ((c == '$' || c == '@') &&
       (c2 == '#'))
     {
       input_get(); // swallow '#'
@@ -621,8 +703,7 @@ lexer::scan (bool expand_args)
       semiskipped_p ++;
       goto semiskip;
     }
-  else if (expand_args &&
-           (c == '$' || c == '@') &&
+  else if ((c == '$' || c == '@') &&
            (isdigit (c2)))
     {
       unsigned idx = 0;
@@ -632,11 +713,12 @@ lexer::scan (bool expand_args)
           idx = (idx * 10) + (c2 - '0');
           c2 = input_peek ();
         } while (c2 > 0 &&
-                 isdigit (c2) && 
+                 isdigit (c2) &&
                  idx <= session.args.size()); // prevent overflow
       if (idx == 0 ||
           idx-1 >= session.args.size())
-          throw parse_error ("command line argument index invalid or out of range", n);
+        throw parse_error ("command line argument index " + lex_cast<string>(idx)
+                           + " out of range [1-" + lex_cast<string>(session.args.size()) + "]", n);
 
       string arg = session.args[idx-1];
       if (c == '$') input_put (arg);
@@ -645,11 +727,13 @@ lexer::scan (bool expand_args)
       goto semiskip;
     }
 
-  else if (isalpha (c) || c == '$' || c == '@' || c == '_')
+  else if (isalpha (c) || c == '$' || c == '@' || c == '_' ||
+	   (wildcard && c == '*'))
     {
       n->type = tok_identifier;
       n->content = (char) c;
-      while (isalnum (c2) || c2 == '_' || c2 == '$')
+      while (isalnum (c2) || c2 == '_' || c2 == '$' ||
+	     (wildcard && c2 == '*'))
 	{
           input_get ();
           n->content.push_back (c2);
@@ -674,7 +758,7 @@ lexer::scan (bool expand_args)
           || n->content == "string"
           || n->content == "long")
         n->type = tok_keyword;
-      
+
       return n;
     }
 
@@ -719,7 +803,7 @@ lexer::scan (bool expand_args)
 	  if (c == '\"') // closing double-quotes
 	    break;
 	  else if (c == '\\') // see also input_put
-	    {	      
+	    {
 	      c = input_get ();
 	      switch (c)
 		{
@@ -733,7 +817,7 @@ lexer::scan (bool expand_args)
 		case '0' ... '7': // NB: need only match the first digit
 		case '\\':
 		  // Pass these escapes through to the string value
-		  // being parsed; it will be emitted into a C literal. 
+		  // being parsed; it will be emitted into a C literal.
 
 		  n->content.push_back('\\');
 
@@ -856,7 +940,7 @@ lexer::scan (bool expand_args)
         {
           n->content = s2;
           input_get (); // swallow other character
-        }   
+        }
       else
         {
           n->content = s1;
@@ -880,6 +964,8 @@ stapfile*
 parser::parse ()
 {
   stapfile* f = new stapfile;
+  input.set_current_file (f);
+  f->file_contents = input.get_input_contents ();
   f->name = input_name;
 
   bool empty = true;
@@ -923,7 +1009,7 @@ parser::parse ()
 	{
 	  print_error (pe);
           if (pe.skip_some) // for recovery
-            try 
+            try
               {
                 // Quietly swallow all tokens until the next '}'.
                 while (1)
@@ -948,15 +1034,18 @@ parser::parse ()
     {
       cerr << "Input file '" << input_name << "' is empty or missing." << endl;
       delete f;
+      input.set_current_file (0);
       return 0;
     }
   else if (num_errors > 0)
     {
       cerr << num_errors << " parse error(s)." << endl;
       delete f;
+      input.set_current_file (0);
       return 0;
     }
-  
+
+  input.set_current_file (0);
   return f;
 }
 
@@ -979,18 +1068,22 @@ parser::parse_probe (std::vector<probe *> & probe_ret,
   while (1)
     {
       probe_point * pp = parse_probe_point ();
-      
+
       const token* t = peek ();
-      if (equals_ok && t 
+      if (equals_ok && t
           && t->type == tok_operator && t->content == "=")
         {
+          if (pp->optional || pp->sufficient)
+            throw parse_error ("probe point alias name cannot be optional nor sufficient", pp->tok);
           aliases.push_back(pp);
           next ();
           continue;
         }
-      else if (equals_ok && t 
+      else if (equals_ok && t
           && t->type == tok_operator && t->content == "+=")
         {
+          if (pp->optional || pp->sufficient)
+            throw parse_error ("probe point alias name cannot be optional nor sufficient", pp->tok);
           aliases.push_back(pp);
           epilogue_alias = 1;
           next ();
@@ -1110,7 +1203,7 @@ parser::parse_statement ()
       n->tok = next ();
       return n;
     }
-  else if (t && t->type == tok_operator && t->content == "{")  
+  else if (t && t->type == tok_operator && t->content == "{")
     return parse_stmt_block ();
   else if (t && t->type == tok_keyword && t->content == "if")
     return parse_if_statement ();
@@ -1158,7 +1251,7 @@ parser::parse_global (vector <vardecl*>& globals, vector<probe*>&)
       for (unsigned i=0; i<globals.size(); i++)
 	if (globals[i]->name == t->content)
 	  throw parse_error ("duplicate global name");
-      
+
       vardecl* d = new vardecl;
       d->name = t->content;
       d->tok = t;
@@ -1189,6 +1282,9 @@ parser::parse_global (vector <vardecl*>& globals, vector<probe*>&)
 	  t = peek ();
 	}
 
+      if (t && t->type == tok_operator && t->content == ";") // termination
+	  next();
+
       if (t && t->type == tok_operator && t->content == ",") // next global
 	{
 	  next ();
@@ -1261,7 +1357,7 @@ parser::parse_functiondecl (std::vector<functiondecl*>& functions)
 	  else if (t->type == tok_keyword && t->content == "long")
 	    vd->type = pe_long;
 	  else throw parse_error ("expected 'string' or 'long'");
-	  
+
 	  t = next ();
 	}
       if (t->type == tok_operator && t->content == ")")
@@ -1289,11 +1385,10 @@ parser::parse_probe_point ()
 
   while (1)
     {
-      const token* t = next ();
+      const token* t = next (true); // wildcard scanning here
       if (! (t->type == tok_identifier
 	     // we must allow ".return" and ".function", which are keywords
-	     || t->type == tok_keyword
-	     || (t->type == tok_operator && t->content == "*")))
+	     || t->type == tok_keyword))
         throw parse_error ("expected identifier or '*'");
 
       if (pl->tok == 0) pl->tok = t;
@@ -1303,28 +1398,8 @@ parser::parse_probe_point ()
       pl->components.push_back (c);
       // NB we may add c->arg soon
 
-      const token* last_t = t;
       t = peek ();
 
-      // We need to keep going until we find something other than a
-      // '*' or identifier, since a probe point wildcard can be
-      // something like "*a", "*a*", "a*b", "a*b*", etc.
-      while (t &&
-	     // case 1: '*{identifier}'
-	     ((last_t->type == tok_operator && last_t->content == "*"
-	       && (t->type == tok_identifier || t->type == tok_keyword))
-	      // case 2: '{identifier}*'
-	      || ((last_t->type == tok_identifier
-		   || last_t->type == tok_keyword)
-		  && t->type == tok_operator && t->content == "*")))
-        {
-	  c->functor += t->content;
-	  next ();			// consume the identifier or '*'
-
-	  last_t = t;
-	  t = peek ();
-	}
-
       // consume optional parameter
       if (t && t->type == tok_operator && t->content == "(")
         {
@@ -1362,25 +1437,25 @@ parser::parse_probe_point ()
         {
           next ();
           t = peek ();
-          if (! (t->type == tok_operator && t->content == "("))
+          if (t && ! (t->type == tok_operator && t->content == "("))
             throw parse_error ("expected '('");
           next ();
 
           pl->condition = parse_expression ();
 
           t = peek ();
-          if (! (t->type == tok_operator && t->content == ")"))
+          if (t && ! (t->type == tok_operator && t->content == ")"))
             throw parse_error ("expected ')'");
           next ();
           t = peek ();
           // fall through
         }
 
-      if (t && t->type == tok_operator 
+      if (t && t->type == tok_operator
           && (t->content == "{" || t->content == "," ||
               t->content == "=" || t->content == "+=" ))
         break;
-      
+
       throw parse_error ("expected one of '. , ( ? ! { = +='");
     }
 
@@ -1419,7 +1494,7 @@ parser::parse_literal ()
 	      || (neg && (unsigned long long) value > 9223372036854775808ULL)
 	      || (unsigned long long) value > 18446744073709551615ULL
 	      || value < -9223372036854775807LL-1)
-	    throw parse_error ("number invalid or out of range"); 
+	    throw parse_error ("number invalid or out of range");
 
 	  if (neg)
 	    value = -value;
@@ -1589,7 +1664,7 @@ parser::parse_for_loop ()
       if (! (t->type == tok_operator && t->content == ";"))
 	throw parse_error ("expected ';'");
     }
-  
+
   // increment + ")"
   t = peek ();
   if (t && t->type == tok_operator && t->content == ")")
@@ -1635,7 +1710,7 @@ parser::parse_while_loop ()
   t = next ();
   if (! (t->type == tok_operator && t->content == ")"))
     throw parse_error ("expected ')'");
-  
+
   // block
   s->block = parse_statement ();
 
@@ -1702,7 +1777,7 @@ parser::parse_foreach_loop ()
               next ();
               break;
             }
-          else 
+          else
             throw parse_error ("expected ',' or ']'");
         }
       else
@@ -1712,7 +1787,7 @@ parser::parse_foreach_loop ()
   t = next ();
   if (! (t->type == tok_keyword && t->content == "in"))
     throw parse_error ("expected 'in'");
- 
+
   s->base = parse_indexable();
 
   t = peek ();
@@ -1756,7 +1831,7 @@ parser::parse_assignment ()
 
   const token* t = peek ();
   // right-associative operators
-  if (t && t->type == tok_operator 
+  if (t && t->type == tok_operator
       && (t->content == "=" ||
 	  t->content == "<<<" ||
 	  t->content == "+=" ||
@@ -1770,7 +1845,7 @@ parser::parse_assignment ()
 	  t->content == "^=" ||
 	  t->content == "|=" ||
 	  t->content == ".=" ||
-	  false)) 
+	  false))
     {
       // NB: lvalueness is checked during elaboration / translation
       assignment* e = new assignment;
@@ -1816,7 +1891,7 @@ expression*
 parser::parse_logical_or ()
 {
   expression* op1 = parse_logical_and ();
-  
+
   const token* t = peek ();
   while (t && t->type == tok_operator && t->content == "||")
     {
@@ -1955,7 +2030,7 @@ parser::parse_array_in ()
               next ();
               break;
             }
-          else 
+          else
             throw parse_error ("expected ',' or ']'");
         }
       else
@@ -1989,7 +2064,7 @@ parser::parse_comparison ()
   expression* op1 = parse_shift ();
 
   const token* t = peek ();
-  while (t && t->type == tok_operator 
+  while (t && t->type == tok_operator
       && (t->content == ">" ||
           t->content == "<" ||
           t->content == "==" ||
@@ -2017,7 +2092,7 @@ parser::parse_shift ()
   expression* op1 = parse_concatenation ();
 
   const token* t = peek ();
-  while (t && t->type == tok_operator && 
+  while (t && t->type == tok_operator &&
          (t->content == "<<" || t->content == ">>"))
     {
       binary_expression* e = new binary_expression;
@@ -2064,7 +2139,7 @@ parser::parse_additive ()
   expression* op1 = parse_multiplicative ();
 
   const token* t = peek ();
-  while (t && t->type == tok_operator 
+  while (t && t->type == tok_operator
       && (t->content == "+" || t->content == "-"))
     {
       binary_expression* e = new binary_expression;
@@ -2087,7 +2162,7 @@ parser::parse_multiplicative ()
   expression* op1 = parse_unary ();
 
   const token* t = peek ();
-  while (t && t->type == tok_operator 
+  while (t && t->type == tok_operator
       && (t->content == "*" || t->content == "/" || t->content == "%"))
     {
       binary_expression* e = new binary_expression;
@@ -2108,9 +2183,9 @@ expression*
 parser::parse_unary ()
 {
   const token* t = peek ();
-  if (t && t->type == tok_operator 
-      && (t->content == "+" || 
-          t->content == "-" || 
+  if (t && t->type == tok_operator
+      && (t->content == "+" ||
+          t->content == "-" ||
           t->content == "!" ||
           t->content == "~" ||
           false))
@@ -2137,7 +2212,7 @@ parser::parse_crement () // as in "increment" / "decrement"
   // cases like "4++".
 
   const token* t = peek ();
-  if (t && t->type == tok_operator 
+  if (t && t->type == tok_operator
       && (t->content == "++" || t->content == "--"))
     {
       pre_crement* e = new pre_crement;
@@ -2150,9 +2225,9 @@ parser::parse_crement () // as in "increment" / "decrement"
 
   // post-crement or non-crement
   expression *op1 = parse_value ();
-  
+
   t = peek ();
-  if (t && t->type == tok_operator 
+  if (t && t->type == tok_operator
       && (t->content == "++" || t->content == "--"))
     {
       post_crement* e = new post_crement;
@@ -2241,7 +2316,7 @@ parser::parse_indexable ()
 
 // var, indexable[index], func(parms), printf("...", ...), $var, $var->member, @stat_op(stat)
 expression*
-parser::parse_symbol () 
+parser::parse_symbol ()
 {
   hist_op *hop = NULL;
   symbol *sym = NULL;
@@ -2250,7 +2325,7 @@ parser::parse_symbol ()
 
   if (!hop)
     {
-      // If we didn't get a hist_op, then we did get an identifier. We can 
+      // If we didn't get a hist_op, then we did get an identifier. We can
       // now scrutinize this identifier for the various magic forms of identifier
       // (printf, @stat_op, and $var...)
 
@@ -2277,7 +2352,7 @@ parser::parse_symbol ()
 	  expect_op(")");
 	  return sop;
 	}
-      
+
       else if (print_format::parse_print(name,
 	 pf_stream, pf_format, pf_delim, pf_newline, pf_char))
 	{
@@ -2298,16 +2373,16 @@ parser::parse_symbol ()
 	      // construct. This is sort of gross but it avoids
 	      // promoting histogram references to typeful
 	      // expressions.
-	      
+
 	      hop = NULL;
 	      t = parse_hist_op_or_bare_name(hop, name);
 	      assert(hop);
-	      
+
 	      // It is, sadly, possible that even while parsing a
 	      // hist_op, we *mis-guessed* and the user wishes to
 	      // print(@hist_op(foo)[bucket]), a scalar. In that case
 	      // we must parse the arrayindex and print an expression.
-	      
+
 	      if (!peek_op ("["))
 		fmt->hist = hop;
 	      else
@@ -2366,7 +2441,7 @@ parser::parse_symbol ()
 	  expect_op(")");
 	  return fmt;
 	}
-      
+
       else if (name.size() > 0 && name[0] == '$')
 	{
 	  // target_symbol time
@@ -2377,20 +2452,20 @@ parser::parse_symbol ()
 	    {
 	      string c;
 	      if (peek_op ("->"))
-		{ 
-		  next(); 
+		{
+		  next();
 		  expect_ident_or_keyword (c);
 		  tsym->components.push_back
 		    (make_pair (target_symbol::comp_struct_member, c));
 		}
 	      else if (peek_op ("["))
-		{ 
+		{
 		  next();
 		  expect_unknown (tok_number, c);
 		  expect_op ("]");
 		  tsym->components.push_back
 		    (make_pair (target_symbol::comp_literal_array_index, c));
-		}	    
+		}
 	      else
 		break;
 	    }
@@ -2435,8 +2510,8 @@ parser::parse_symbol ()
 	  sym->tok = t;
 	}
     }
-  
-  // By now, either we had a hist_op in the first place, or else 
+
+  // By now, either we had a hist_op in the first place, or else
   // we had a plain word and it was converted to a symbol.
 
   assert (!hop != !sym); // logical XOR
@@ -2458,9 +2533,9 @@ parser::parse_symbol ()
         {
           ai->indexes.push_back (parse_expression ());
           if (peek_op ("]"))
-            { 
-	      next(); 
-	      break; 
+            {
+	      next();
+	      break;
 	    }
           else if (peek_op (","))
 	    {
@@ -2479,7 +2554,8 @@ parser::parse_symbol ()
 
   if (hop)
     throw parse_error("base histogram operator where expression expected", t);
-  
-  return sym;  
+
+  return sym;
 }
 
+/* vim: set sw=2 ts=8 cino=>4,n-2,{2,^-2,t0,(0,u0,w1,M1 : */
