@@ -1,7 +1,7 @@
 /* -*- linux-c -*-
  *
  * /proc command channels
- * Copyright (C) 2007 Red Hat Inc.
+ * Copyright (C) 2007-2009 Red Hat Inc.
  *
  * This file is part of systemtap, and is free software.  You can
  * redistribute it and/or modify it under the terms of the GNU General
@@ -14,6 +14,9 @@
  * number of needed files.
  */
 
+#ifndef _STP_PROCFS_C_
+#define _STP_PROCFS_C_
+
 #define STP_MAX_PROCFS_FILES 16
 static int _stp_num_pde = 0;
 static int _stp_num_procfs_files = 0;
@@ -22,15 +25,22 @@ static struct proc_dir_entry *_stp_procfs_files[STP_MAX_PROCFS_FILES];
 static struct proc_dir_entry *_stp_proc_stap = NULL;
 static struct proc_dir_entry *_stp_proc_root = NULL;
 
-void _stp_close_procfs(void);
+static void _stp_close_procfs(void);
+
+// 2.6.24 fixed proc_dir_entry refcounting.
+#if LINUX_VERSION_CODE < KERNEL_VERSION(2,6,24)
+#define LAST_ENTRY_COUNT 0
+#else
+#define LAST_ENTRY_COUNT 1
+#endif
 
 /*
  * Removes /proc/systemtap/{module_name} and /proc/systemtap (if empty)
  */
-void _stp_rmdir_proc_module(void)
+static void _stp_rmdir_proc_module(void)
 {
         if (_stp_proc_root && _stp_proc_root->subdir == NULL) {
-		if (atomic_read(&_stp_proc_root->count))
+		if (atomic_read(&_stp_proc_root->count) != LAST_ENTRY_COUNT)
 			_stp_warn("Removal of /proc/systemtap/%s\nis deferred until it is no longer in use.\n"
 				  "Systemtap module removal will block.\n", THIS_MODULE->name);	
 		remove_proc_entry(THIS_MODULE->name, _stp_proc_stap);
@@ -38,7 +48,7 @@ void _stp_rmdir_proc_module(void)
 	}
 
 	if (_stp_proc_stap) {
-		if (!_stp_lock_debugfs()) {
+		if (!_stp_lock_transport_dir()) {
 			errk("Unable to lock transport directory.\n");
 			return;
 		}
@@ -55,7 +65,7 @@ void _stp_rmdir_proc_module(void)
 			_stp_proc_stap = NULL;
 		}
 
-		_stp_unlock_debugfs();
+		_stp_unlock_transport_dir();
 	}
 }
 
@@ -64,12 +74,12 @@ void _stp_rmdir_proc_module(void)
  * Safely creates /proc/systemtap (if necessary) and
  * /proc/systemtap/{module_name}.
  */
-int _stp_mkdir_proc_module(void)
+static int _stp_mkdir_proc_module(void)
 {	
         if (_stp_proc_root == NULL) {
 		struct nameidata nd;
 
-		if (!_stp_lock_debugfs()) {
+		if (!_stp_lock_transport_dir()) {
 			errk("Unable to lock transport directory.\n");
 			goto done;
 		}
@@ -83,7 +93,7 @@ int _stp_mkdir_proc_module(void)
 			/* doesn't exist, so create it */
 			_stp_proc_stap = proc_mkdir ("systemtap", NULL);
 			if (_stp_proc_stap == NULL) {
-				_stp_unlock_debugfs();
+				_stp_unlock_transport_dir();
 				goto done;
 			}
 		} else {
@@ -98,10 +108,12 @@ int _stp_mkdir_proc_module(void)
 		}
 		
 		_stp_proc_root = proc_mkdir(THIS_MODULE->name, _stp_proc_stap);
+#ifdef AUTOCONF_PROCFS_OWNER
 		if (_stp_proc_root != NULL)
 			_stp_proc_root->owner = THIS_MODULE;
+#endif
 
-		_stp_unlock_debugfs();
+		_stp_unlock_transport_dir();
 	}
 done:
 	return (_stp_proc_root) ? 1 : 0;
@@ -121,7 +133,7 @@ static struct proc_dir_entry *_stp_procfs_lookup(const char *dir, struct proc_di
 	return NULL;
 }
 
-int _stp_create_procfs(const char *path, int num)
+static int _stp_create_procfs(const char *path, int num)
 {  
 	const char *p;
 	char *next;
@@ -154,7 +166,9 @@ int _stp_create_procfs(const char *path, int num)
 				    goto err;
 			    }
 			    _stp_pde[_stp_num_pde++] = last_dir;
+#ifdef AUTOCONF_PROCFS_OWNER
 			    last_dir->owner = THIS_MODULE;
+#endif
 			    last_dir->uid = _stp_uid;
 			    last_dir->gid = _stp_gid;
 		} else {
@@ -188,7 +202,7 @@ err:
 	return -1;
 }
 
-void _stp_close_procfs(void)
+static void _stp_close_procfs(void)
 {
 	int i;
 	for (i = _stp_num_pde-1; i >= 0; i--) {
@@ -198,3 +212,5 @@ void _stp_close_procfs(void)
 	_stp_num_pde = 0;
 	_stp_rmdir_proc_module();
 }
+
+#endif	/* _STP_PROCFS_C_ */
