@@ -13,8 +13,8 @@
 static char _stp_stdout[] = "_stdout_";
 
 #define to_oct_digit(c) ((c) + '0')
-void _stp_vsprintf (char *str, const char *fmt, va_list args);
-void _stp_text_str(char *out, char *in, int len, int quoted, int user);
+static void _stp_vsprintf (char *str, const char *fmt, va_list args);
+static void _stp_text_str(char *out, char *in, int len, int quoted, int user);
 
 /*
  * Powerpc uses a paranoid user address check in __get_user() which
@@ -24,7 +24,7 @@ void _stp_text_str(char *out, char *in, int len, int quoted, int user);
  * to __get_user() if not. Other archs can use __get_user() as is
  */
 #ifdef __powerpc64__
-#ifdef __get_user_inatomic(x, ptr)
+#ifdef __get_user_inatomic
 #define __stp_get_user(x, ptr) __get_user_inatomic(x, ptr)
 #else /* __get_user_inatomic */
 #define __stp_get_user(x, ptr) __get_user(x, ptr)
