@@ -14,7 +14,7 @@ static int _stp_current_buffers = STP_DEFAULT_BUFFERS;
 
 static _stp_mempool_t *_stp_pool_q;
 static struct list_head _stp_ctl_ready_q;
-DEFINE_SPINLOCK(_stp_ctl_ready_lock);
+static DEFINE_SPINLOCK(_stp_ctl_ready_lock);
 
 static ssize_t _stp_ctl_write_cmd(struct file *file, const char __user *buf, size_t count, loff_t *ppos)
 {
@@ -30,6 +30,7 @@ static ssize_t _stp_ctl_write_cmd(struct file *file, const char __user *buf, siz
 	count -= sizeof(u32);
 	buf += sizeof(u32);
 
+
 #ifdef DEBUG_TRANS
 	if (type < STP_MAX_CMD)
 		_dbug("Got %s. len=%d\n", _stp_command_name[type], (int)count);
@@ -56,9 +57,11 @@ static ssize_t _stp_ctl_write_cmd(struct file *file, const char __user *buf, siz
 #else
 		return -1;
 #endif
+	case STP_RELOCATION:
+          	_stp_do_relocation (buf, count);
+          	break;
+
 	case STP_READY:
-		/* request symbolic information */
-		/*  _stp_ask_for_symbols(); */
 		break;
 
 	default:
@@ -66,7 +69,7 @@ static ssize_t _stp_ctl_write_cmd(struct file *file, const char __user *buf, siz
 		return -EINVAL;
 	}
 
-	return count;
+	return count; /* Pretend that we absorbed the entire message. */
 }
 
 struct _stp_buffer {
