@@ -2,7 +2,7 @@
  * transport.c - stp transport functions
  *
  * Copyright (C) IBM Corporation, 2005
- * Copyright (C) Red Hat Inc, 2005-2008
+ * Copyright (C) Red Hat Inc, 2005-2009
  * Copyright (C) Intel Corporation, 2006
  *
  * This file is part of systemtap, and is free software.  You can
@@ -20,14 +20,13 @@
 #include "time.c"
 #include "../mempool.c"
 #include "symbols.c"
-#include "../procfs.c"
 
 static struct utt_trace *_stp_utt = NULL;
 static unsigned int utt_seq = 1;
 static int _stp_probes_started = 0;
-pid_t _stp_target = 0;
+static pid_t _stp_target = 0;
 static int _stp_exit_called = 0;
-int _stp_exit_flag = 0;
+static int _stp_exit_flag = 0;
 #ifdef STP_OLD_TRANSPORT
 #include "relayfs.c"
 #include "procfs.c"
@@ -42,9 +41,9 @@ module_param(_stp_bufsize, int, 0);
 MODULE_PARM_DESC(_stp_bufsize, "buffer size");
 
 /* forward declarations */
-void probe_exit(void);
-int probe_start(void);
-void _stp_exit(void);
+static void probe_exit(void);
+static int probe_start(void);
+static void _stp_exit(void);
 
 /* check for new workqueue API */
 #ifdef DECLARE_DELAYED_WORK
@@ -61,9 +60,18 @@ static struct workqueue_struct *_stp_wq;
  *	_stp_handle_start - handle STP_START
  */
 
-void _stp_handle_start(struct _stp_msg_start *st)
+static void _stp_handle_start(struct _stp_msg_start *st)
 {
 	dbug_trans(1, "stp_handle_start\n");
+
+#ifdef STAPCONF_VM_AREA
+        { /* PR9740: workaround for kernel valloc bug. */
+                void *dummy;
+                dummy = alloc_vm_area (PAGE_SIZE);
+                free_vm_area (dummy);
+        }
+#endif
+
 	_stp_target = st->target;
 	st->res = probe_start();
 	if (st->res >= 0)
@@ -79,13 +87,11 @@ void _stp_handle_start(struct _stp_msg_start *st)
 /* when someone does /sbin/rmmod on a loaded systemtap module. */
 static void _stp_cleanup_and_exit(int send_exit)
 {
-	dbug_trans(1, "cleanup_and_exit (%d)\n", send_exit);
 	if (!_stp_exit_called) {
 		int failures;
 
+                dbug_trans(1, "cleanup_and_exit (%d)\n", send_exit);
 		_stp_exit_flag = 1;
-		unregister_module_notifier(&_stp_module_load_nb);
-
 		/* we only want to do this stuff once */
 		_stp_exit_called = 1;
 
@@ -160,9 +166,9 @@ static void _stp_work_queue(void *data)
 		wake_up_interruptible(&_stp_ctl_wq);
 
 	/* if exit flag is set AND we have finished with probe_start() */
-	if (unlikely(_stp_exit_flag))
+	if (unlikely(_stp_exit_flag && _stp_probes_started))
 		_stp_cleanup_and_exit(1);
-	else if (likely(_stp_attached))
+	if (likely(_stp_attached))
 		queue_delayed_work(_stp_wq, &_stp_work, STP_WORK_TIMER);
 }
 
@@ -172,7 +178,7 @@ static void _stp_work_queue(void *data)
  *	This is called automatically when the module is unloaded.
  *     
  */
-void _stp_transport_close()
+static void _stp_transport_close()
 {
 	dbug_trans(1, "%d: ************** transport_close *************\n", current->pid);
 	_stp_cleanup_and_exit(0);
@@ -180,8 +186,7 @@ void _stp_transport_close()
 	_stp_unregister_ctl_channel();
 	if (_stp_utt)
 		utt_trace_remove(_stp_utt);
-	_stp_free_modules();
-	_stp_kill_time();
+	_stp_kill_time();  /* Go to a beach.  Drink a beer.  */
 	_stp_print_cleanup();	/* free print buffers */
 	_stp_mem_debug_done();
 	dbug_trans(1, "---- CLOSED ----\n");
@@ -208,14 +213,19 @@ static struct utt_trace *_stp_utt_open(void)
  * _stp_transport_init() is called from the module initialization.
  *   It does the bare minimum to exchange commands with staprun 
  */
-int _stp_transport_init(void)
+static int _stp_transport_init(void)
 {
 	int ret;
 
 	dbug_trans(1, "transport_init\n");
 	_stp_init_pid = current->pid;
+#ifdef STAPCONF_TASK_UID
 	_stp_uid = current->uid;
 	_stp_gid = current->gid;
+#else
+	_stp_uid = current_uid();
+	_stp_gid = current_gid();
+#endif
 
 #ifdef RELAY_GUEST
 	/* Guest scripts use relay only for reporting warnings and errors */
@@ -263,20 +273,11 @@ int _stp_transport_init(void)
 	
 	_stp_transport_state = 1;
 
-	dbug_trans(1, "calling init_kernel_symbols\n");
-	if (_stp_init_kernel_symbols() < 0)
-		goto err4;
-	
-	dbug_trans(1, "calling init_modules\n");
-	if (_stp_init_modules() < 0)
-		goto err4;
+        /* Signal stapio to send us STP_START back (XXX: ?!?!?!).  */
+	_stp_ctl_send(STP_TRANSPORT, NULL, 0);
 
 	return 0;
 
-err4:
-	errk("failed to initialize modules\n");
-	_stp_free_modules();
-	destroy_workqueue(_stp_wq);
 err3:
 	_stp_print_cleanup();
 err2:
@@ -309,7 +310,7 @@ static inline void _stp_unlock_inode(struct inode *inode)
 
 static struct dentry *_stp_lockfile = NULL;
 
-static int _stp_lock_debugfs(void)
+static int _stp_lock_transport_dir(void)
 {
 	int numtries = 0;
 #ifdef STP_OLD_TRANSPORT
@@ -324,7 +325,7 @@ static int _stp_lock_debugfs(void)
 	return 1;
 }
 
-static void _stp_unlock_debugfs(void)
+static void _stp_unlock_transport_dir(void)
 {
 	if (_stp_lockfile) {
 #ifdef STP_OLD_TRANSPORT
@@ -357,7 +358,7 @@ static struct dentry *_stp_get_root_dir(const char *name)
 		return NULL;
 	}
 
-	if (!_stp_lock_debugfs()) {
+	if (!_stp_lock_transport_dir()) {
 		errk("Couldn't lock transport directory.\n");
 		return NULL;
 	}
@@ -379,7 +380,7 @@ static struct dentry *_stp_get_root_dir(const char *name)
 			errk("Could not create or find transport directory.\n");
 		}
 	}
-	_stp_unlock_debugfs();
+	_stp_unlock_transport_dir();
 	return root;
 }
 
