@@ -108,7 +108,6 @@ static void _stp_create_unwind_hdr(struct _stp_module *m)
 		header = _stp_vmalloc(hdrSize);
 		if (header == NULL)
 			return;
-		m->allocated.unwind_hdr = 1;
 	}
 
 	header->version = 1;
@@ -156,17 +155,10 @@ static void _stp_create_unwind_hdr(struct _stp_module *m)
 bad:
 	dbug_unwind(1, "unwind data for %s is unacceptable. Freeing.", m->name);
 	if (header) {
-		if (m->allocated.unwind_hdr) {
-			m->allocated.unwind_hdr = 0;
-			_stp_vfree(header);
-		} else
-			_stp_kfree(header);
+          _stp_vfree(header);
 	}
 	if (m->unwind_data) {
-		if (m->allocated.unwind_data)
-			_stp_vfree(m->unwind_data);
-		else
-			_stp_kfree(m->unwind_data);
+          _stp_vfree(m->unwind_data);
 		m->unwind_data = NULL;
 		m->unwind_data_len = 0;
 	}
@@ -558,10 +550,29 @@ static int processCFI(const u8 *start, const u8 *end, unsigned long targetLoc, s
 	return result && ptr.p8 == end && (targetLoc == 0 || state->label == NULL);
 }
 
+// If this is an address inside a module, adjust for section relocation
+// and the elfutils base relocation done during loading of the .dwarf_frame
+// in translate.cxx.
+static unsigned long
+adjustStartLoc (unsigned long startLoc,
+		struct _stp_module *m,
+		struct _stp_section *s)
+{
+  if (startLoc && (strcmp (m->name, "kernel") != 0))
+    {
+      startLoc = _stp_module_relocate (m->name, s->name,
+				       startLoc);
+      startLoc -= m->dwarf_module_base;
+    }
+  return startLoc;
+}
+
 /* If we previously created an unwind header, then use it now to binary search */
 /* for the FDE corresponding to pc. */
 
-static u32 *_stp_search_unwind_hdr(unsigned long pc, struct _stp_module *m)
+static u32 *_stp_search_unwind_hdr(unsigned long pc,
+				   struct _stp_module *m,
+				   struct _stp_section *s)
 {
 	const u8 *ptr, *end, *hdr = m->unwind_hdr;
 	unsigned long startLoc;
@@ -608,6 +619,7 @@ static u32 *_stp_search_unwind_hdr(unsigned long pc, struct _stp_module *m)
 	do {
 		const u8 *cur = ptr + (num / 2) * (2 * tableSize);
 		startLoc = read_pointer(&cur, cur + tableSize, hdr[3]);
+		startLoc = adjustStartLoc(startLoc, m, s);
 		if (pc < startLoc)
 			num /= 2;
 		else {
@@ -616,7 +628,7 @@ static u32 *_stp_search_unwind_hdr(unsigned long pc, struct _stp_module *m)
 		}
 	} while (startLoc && num > 1);
 
-	if (num == 1 && (startLoc = read_pointer(&ptr, ptr + tableSize, hdr[3])) != 0 && pc >= startLoc)
+	if (num == 1 && (startLoc = adjustStartLoc(read_pointer(&ptr, ptr + tableSize, hdr[3]), m, s)) != 0 && pc >= startLoc)
 		fde = (void *)read_pointer(&ptr, ptr + tableSize, hdr[3]);
 
 	dbug_unwind(1, "returning fde=%lx startLoc=%lx", fde, startLoc);
@@ -643,7 +655,7 @@ static const char *_stp_enc_lo_name[] = {
 	"DW_EH_PE_sdata4",
 	"DW_EH_PE_sdata8"
 };
-char *_stp_eh_enc_name(signed type)
+static char *_stp_eh_enc_name(signed type)
 {
 	static char buf[64];
 	int hi, low;
@@ -673,7 +685,7 @@ char *_stp_eh_enc_name(signed type)
 /* Unwind to previous to frame.  Returns 0 if successful, negative
  * number in case of an error.  A positive return means unwinding is finished; 
  * don't try to fallback to dumping addresses on the stack. */
-int unwind(struct unwind_frame_info *frame)
+static int unwind(struct unwind_frame_info *frame)
 {
 #define FRAME_REG(r, t) (((t *)frame)[reg_info[r].offs])
 	const u32 *fde, *cie = NULL;
@@ -684,6 +696,7 @@ int unwind(struct unwind_frame_info *frame)
 	signed ptrType = -1;
 	uleb128_t retAddrReg = 0;
 	struct _stp_module *m;
+	struct _stp_section *s = NULL;
 	struct unwind_state state;
 
 	dbug_unwind(1, "pc=%lx, %lx", pc, UNW_PC(frame));
@@ -691,7 +704,7 @@ int unwind(struct unwind_frame_info *frame)
 	if (UNW_PC(frame) == 0)
 		return -EINVAL;
 
-	m = _stp_get_unwind_info(pc);
+	m = _stp_mod_sec_lookup (pc, &s);
 	if (unlikely(m == NULL)) {
 		dbug_unwind(1, "No module found for pc=%lx", pc);
 		return -EINVAL;
@@ -702,7 +715,7 @@ int unwind(struct unwind_frame_info *frame)
 		goto err;
 	}
 
-	fde = _stp_search_unwind_hdr(pc, m);
+	fde = _stp_search_unwind_hdr(pc, m, s);
 	dbug_unwind(1, "%s: fde=%lx\n", m->name, fde);
 
 	/* found the fde, now set startLoc and endLoc */
@@ -712,6 +725,8 @@ int unwind(struct unwind_frame_info *frame)
 			ptr = (const u8 *)(fde + 2);
 			ptrType = fde_pointer_type(cie);
 			startLoc = read_pointer(&ptr, (const u8 *)(fde + 1) + *fde, ptrType);
+			startLoc = adjustStartLoc(startLoc, m, s);
+
 			dbug_unwind(2, "startLoc=%lx, ptrType=%s", startLoc, _stp_eh_enc_name(ptrType));
 			if (!(ptrType & DW_EH_PE_indirect))
 				ptrType &= DW_EH_PE_FORM | DW_EH_PE_signed;
@@ -741,6 +756,7 @@ int unwind(struct unwind_frame_info *frame)
 
 			ptr = (const u8 *)(fde + 2);
 			startLoc = read_pointer(&ptr, (const u8 *)(fde + 1) + *fde, ptrType);
+			startLoc = adjustStartLoc(startLoc, m, s);
 			dbug_unwind(2, "startLoc=%lx, ptrType=%s", startLoc, _stp_eh_enc_name(ptrType));
 			if (!startLoc)
 				continue;
@@ -940,21 +956,18 @@ int unwind(struct unwind_frame_info *frame)
 			break;
 		}
 	}
-	read_unlock(&m->lock);
 	dbug_unwind(1, "returning 0 (%lx)\n", UNW_PC(frame));
 	return 0;
 
 copy_failed:
 	dbug_unwind(1, "_stp_read_address failed to access memory\n");
 err:
-	read_unlock(&m->lock);
 	return -EIO;
 	
 done:
 	/* PC was in a range convered by a module but no unwind info */
 	/* found for the specific PC. This seems to happen only for kretprobe */
 	/* trampolines and at the end of interrupt backtraces. */
-	read_unlock(&m->lock);
 	return 1;
 #undef CASES
 #undef FRAME_REG
