@@ -32,6 +32,7 @@ struct be_derived_probe_group;
 struct dwarf_derived_probe_group;
 struct uprobe_derived_probe_group;
 struct utrace_derived_probe_group;
+struct itrace_derived_probe_group;
 struct task_finder_derived_probe_group;
 struct timer_derived_probe_group;
 struct profile_derived_probe_group;
@@ -51,16 +52,16 @@ struct module_cache;
 struct statistic_decl
 {
   statistic_decl()
-    : type(none), 
+    : type(none),
       linear_low(0), linear_high(0), linear_step(0)
-  {}    
+  {}
   enum { none, linear, logarithmic } type;
   int64_t linear_low;
   int64_t linear_high;
   int64_t linear_step;
   bool operator==(statistic_decl const & other)
   {
-    return type == other.type 
+    return type == other.type
       && linear_low == other.linear_low
       && linear_high == other.linear_high
       && linear_step == other.linear_step;
@@ -80,14 +81,17 @@ struct systemtap_session
   std::vector<std::string> args;
   std::string kernel_release;
   std::string kernel_base_release;
+  std::string kernel_build_tree;
   std::string architecture;
   std::string runtime_path;
   std::string data_path;
   std::string module_name;
+  std::string stapconf_name;
   std::string output_file;
   std::string cmd;
   int target_pid;
   int last_pass;
+  unsigned perpass_verbose[5];
   unsigned verbose;
   bool timing;
   bool keep_tmpdir;
@@ -103,11 +107,13 @@ struct systemtap_session
   bool prologue_searching;
   bool tapset_compile_coverage;
   bool need_uprobes;
+  bool load_only; // flight recorder mode
 
   // Cache data
   bool use_cache;
   std::string cache_path;
   std::string hash_path;
+  std::string stapconf_path;
 
   // dwarfless operation
   bool consult_symtab;
@@ -130,7 +136,7 @@ struct systemtap_session
   // resolved globals/functions/probes for the run as a whole
   std::vector<stapfile*> files;
   std::vector<vardecl*> globals;
-  std::vector<functiondecl*> functions;
+  std::map<std::string,functiondecl*> functions;
   std::vector<derived_probe*> probes; // see also *_probes groups below
   std::vector<embeddedcode*> embeds;
   std::map<std::string, statistic_decl> stat_decls;
@@ -146,6 +152,7 @@ struct systemtap_session
   dwarf_derived_probe_group* dwarf_derived_probes;
   uprobe_derived_probe_group* uprobe_derived_probes;
   utrace_derived_probe_group* utrace_derived_probes;
+  itrace_derived_probe_group* itrace_derived_probes;
   task_finder_derived_probe_group* task_finder_derived_probes;
   timer_derived_probe_group* timer_derived_probes;
   profile_derived_probe_group* profile_derived_probes;
@@ -164,7 +171,10 @@ struct systemtap_session
   Dwarf_Addr sym_kprobes_text_end;
   Dwarf_Addr sym_stext;
 
+  // List of libdwfl module names to extract symbol/unwind data for.
+  std::set<std::string> unwindsym_modules;
   struct module_cache* module_cache;
+
   std::set<std::string> seen_errors;
   std::set<std::string> seen_warnings;
   unsigned num_errors () { return seen_errors.size(); }
@@ -173,6 +183,7 @@ struct systemtap_session
   const token* last_token;
   void print_token (std::ostream& o, const token* tok);
   void print_error (const semantic_error& e);
+  void print_error_source (std::ostream&, std::string&, const token* tok);
   void print_warning (const std::string& w, const token* tok = 0);
 
   // reNB: new POD members likely need to be explicitly cleared in the ctor.
@@ -183,3 +194,5 @@ struct systemtap_session
 extern int pending_interrupts;
 
 #endif // SESSION_H
+
+/* vim: set sw=2 ts=8 cino=>4,n-2,{2,^-2,t0,(0,u0,w1,M1 : */
