@@ -1,5 +1,5 @@
 // parse tree functions
-// Copyright (C) 2005, 2006 Red Hat Inc.
+// Copyright (C) 2005-2008 Red Hat Inc.
 //
 // This file is part of systemtap, and is free software.  You can
 // redistribute it and/or modify it under the terms of the GNU General
@@ -15,6 +15,10 @@
 #include <typeinfo>
 #include <sstream>
 #include <cassert>
+#include <cstring>
+#include <vector>
+#include <algorithm>
+#include <cstring>
 
 using namespace std;
 
@@ -23,7 +27,7 @@ using namespace std;
 expression::expression ():
   type (pe_unknown), tok (0)
 {
-} 
+}
 
 
 expression::~expression ()
@@ -34,7 +38,7 @@ expression::~expression ()
 statement::statement ():
   tok (0)
 {
-} 
+}
 
 
 statement::~statement ()
@@ -64,7 +68,7 @@ symboldecl::symboldecl ():
   tok (0),
   type (pe_unknown)
 {
-} 
+}
 
 
 symboldecl::~symboldecl ()
@@ -73,12 +77,21 @@ symboldecl::~symboldecl ()
 
 probe_point::probe_point (std::vector<component*> const & comps,
 			  const token * t):
-  components(comps), tok(t), optional (false)
+  components(comps), tok(t), optional (false), sufficient (false),
+  condition (0)
 {
 }
 
+// NB: shallow-copy of compoonents & condition!
+probe_point::probe_point (const probe_point& pp):
+  components(pp.components), tok(pp.tok), optional (pp.optional), sufficient (pp.sufficient),
+  condition (pp.condition)
+{
+}
+
+
 probe_point::probe_point ():
-  tok (0), optional (false)
+  tok (0), optional (false), sufficient (false), condition (0)
 {
 }
 
@@ -97,8 +110,8 @@ probe_point::component::component ():
 }
 
 
-probe_point::component::component (std::string const & f, literal * a): 
-  functor(f), arg(a) 
+probe_point::component::component (std::string const & f, literal * a):
+  functor(f), arg(a)
 {
 }
 
@@ -127,7 +140,7 @@ vardecl::set_arity (int a)
     }
 }
 
-bool 
+bool
 vardecl::compatible_arity (int a)
 {
   if (a == 0 && maxsize > 0)
@@ -203,15 +216,15 @@ void literal_number::print (ostream& o) const
 
 void binary_expression::print (ostream& o) const
 {
-  o << "(" << *left << ") " 
-    << op 
+  o << "(" << *left << ") "
+    << op
     << " (" << *right << ")";
 }
 
 
 void unary_expression::print (ostream& o) const
 {
-  o << op << '(' << *operand << ")"; 
+  o << op << '(' << *operand << ")";
 }
 
 void array_in::print (ostream& o) const
@@ -228,7 +241,7 @@ void array_in::print (ostream& o) const
 
 void post_crement::print (ostream& o) const
 {
-  o << '(' << *operand << ")" << op; 
+  o << '(' << *operand << ")" << op;
 }
 
 
@@ -236,7 +249,7 @@ void ternary_expression::print (ostream& o) const
 {
   o << "(" << *cond << ")?("
     << *truevalue << "):("
-    << *falsevalue << ")"; 
+    << *falsevalue << ")";
 }
 
 
@@ -333,10 +346,59 @@ void functioncall::print (ostream& o) const
   for (unsigned i=0; i<args.size(); i++)
     o << (i>0 ? ", " : "") << *args[i];
   o << ")";
-}  
+}
 
 
-string 
+bool
+print_format::parse_print(const std::string &name,
+  bool &stream, bool &format, bool &delim, bool &newline, bool &_char)
+{
+  const char *n = name.c_str();
+
+  stream = true;
+  format = delim = newline = _char = false;
+
+  if (strcmp(n, "print_char") == 0)
+    {
+      _char = true;
+      return true;
+    }
+
+  if (*n == 's')
+    {
+      stream = false;
+      ++n;
+    }
+
+  if (0 != strncmp(n, "print", 5))
+    return false;
+  n += 5;
+
+  if (*n == 'f')
+    {
+      format = true;
+      ++n;
+    }
+  else
+    {
+      if (*n == 'd')
+        {
+	  delim = true;
+	  ++n;
+	}
+
+      if (*n == 'l' && *(n+1) == 'n')
+        {
+	  newline = true;
+	  n += 2;
+	}
+    }
+
+  return (*n == '\0');
+}
+
+
+string
 print_format::components_to_string(vector<format_component> const & components)
 {
   ostringstream oss;
@@ -380,18 +442,26 @@ print_format::components_to_string(vector<format_component> const & components)
 	  if (i->flags & static_cast<unsigned long>(fmt_flag_special))
 	    oss << '#';
 
-	  if (i->width > 0)
+	  if (i->widthtype == width_dynamic)
+	    oss << '*';
+	  else if (i->widthtype != width_unspecified && i->width > 0)
 	    oss << i->width;
 
-	  if (i->precision > 0)
+	  if (i->prectype == prec_dynamic)
+	    oss << ".*";
+	  else if (i->prectype != prec_unspecified && i->precision > 0)
 	    oss << '.' << i->precision;
 
-	  switch (i->type)	
+	  switch (i->type)
 	    {
 	    case conv_binary:
 	      oss << "b";
 	      break;
 
+	    case conv_char:
+	      oss << "llc";
+	      break;
+
 	    case conv_signed_decimal:
 	      oss << "lld";
 	      break;
@@ -419,9 +489,13 @@ print_format::components_to_string(vector<format_component> const & components)
 	    case conv_string:
 	      oss << 's';
 	      break;
-	      
-	    case conv_size:
-	      oss << 'n';
+
+	    case conv_memory:
+	      oss << 'm';
+	      break;
+
+	    case conv_memory_hex:
+	      oss << 'M';
 	      break;
 
 	    default:
@@ -431,8 +505,8 @@ print_format::components_to_string(vector<format_component> const & components)
     }
   return oss.str ();
 }
- 
-vector<print_format::format_component> 
+
+vector<print_format::format_component>
 print_format::string_to_components(string const & str)
 {
   format_component curr;
@@ -441,7 +515,7 @@ print_format::string_to_components(string const & str)
   curr.clear();
 
   string::const_iterator i = str.begin();
-  
+
   while (i != str.end())
     {
       if (*i != '%')
@@ -462,7 +536,7 @@ print_format::string_to_components(string const & str)
           i += 2;
 	  continue;
 	}
-      else 
+      else
 	{
 	  assert(*i == '%');
 	  if (curr.type != conv_unspecified)
@@ -474,12 +548,12 @@ print_format::string_to_components(string const & str)
 	    }
 	}
       ++i;
-      
+
       if (i == str.end())
 	break;
 
-      // Now we are definitely parsing a conversion. 
-      // Begin by parsing flags (whicih are optional).
+      // Now we are definitely parsing a conversion.
+      // Begin by parsing flags (which are optional).
 
       switch (*i)
 	{
@@ -487,22 +561,22 @@ print_format::string_to_components(string const & str)
 	  curr.flags |= static_cast<unsigned long>(fmt_flag_zeropad);
 	  ++i;
 	  break;
-	  
+
 	case '+':
 	  curr.flags |= static_cast<unsigned long>(fmt_flag_plus);
 	  ++i;
 	  break;
-	  
+
 	case '-':
 	  curr.flags |= static_cast<unsigned long>(fmt_flag_left);
 	  ++i;
 	  break;
-	  
+
 	case ' ':
 	  curr.flags |= static_cast<unsigned long>(fmt_flag_space);
 	  ++i;
 	  break;
-	  
+
 	case '#':
 	  curr.flags |= static_cast<unsigned long>(fmt_flag_special);
 	  ++i;
@@ -512,14 +586,27 @@ print_format::string_to_components(string const & str)
 	  break;
 	}
 
+      if (i == str.end())
+	break;
+
       // Parse optional width
-	  
-      while (i != str.end() && isdigit(*i))
+      if (*i == '*')
 	{
-	  curr.width *= 10;
-	  curr.width += (*i - '0');
+	  curr.widthtype = width_dynamic;
 	  ++i;
 	}
+      else if (isdigit(*i))
+	{
+	  curr.widthtype = width_static;
+	  curr.width = 0;
+	  do
+	    {
+	      curr.width *= 10;
+	      curr.width += (*i - '0');
+	      ++i;
+	    }
+	  while (i != str.end() && isdigit(*i));
+	}
 
       if (i == str.end())
 	break;
@@ -530,42 +617,65 @@ print_format::string_to_components(string const & str)
 	  ++i;
 	  if (i == str.end())
 	    break;
-	  while (i != str.end() && isdigit(*i))
+	  if (*i == '*')
 	    {
-	      curr.precision *= 10;
-	      curr.precision += (*i - '0');
+	      curr.prectype = prec_dynamic;
 	      ++i;
 	    }
+	  else if (isdigit(*i))
+	    {
+	      curr.prectype = prec_static;
+	      curr.precision = 0;
+	      do
+		{
+		  curr.precision *= 10;
+		  curr.precision += (*i - '0');
+		  ++i;
+		}
+	      while (i != str.end() && isdigit(*i));
+	    }
 	}
 
       if (i == str.end())
 	break;
 
-      // Parse the actual conversion specifier (sdiouxX)
+      // Parse the actual conversion specifier (bcsmdioupxXn)
       switch (*i)
 	{
 	  // Valid conversion types
 	case 'b':
 	  curr.type = conv_binary;
 	  break;
-	  
+
+	case 'c':
+	  curr.type = conv_char;
+	  break;
+
 	case 's':
 	  curr.type = conv_string;
 	  break;
-	  
+
+	case 'm':
+	  curr.type = conv_memory;
+	  break;
+
+	case 'M':
+	  curr.type = conv_memory_hex;
+	  break;
+
 	case 'd':
 	case 'i':
 	  curr.type = conv_signed_decimal;
 	  break;
-	  
+
 	case 'o':
 	  curr.type = conv_unsigned_octal;
 	  break;
-	  
+
 	case 'u':
 	  curr.type = conv_unsigned_decimal;
 	  break;
-	  
+
 	case 'p':
 	  curr.type = conv_unsigned_ptr;
 	  break;
@@ -573,32 +683,28 @@ print_format::string_to_components(string const & str)
 	case 'X':
 	  curr.type = conv_unsigned_uppercase_hex;
 	  break;
-	  
+
 	case 'x':
 	  curr.type = conv_unsigned_lowercase_hex;
 	  break;
 
-	case 'n':
-	  curr.type = conv_size;
-	  break;
-	  
 	default:
 	  break;
 	}
-      
+
       if (curr.type == conv_unspecified)
 	throw parse_error("invalid or missing conversion specifier");
-      
+
       ++i;
       res.push_back(curr);
-      curr.clear();      
+      curr.clear();
     }
 
   // If there's a remaining partly-composed conversion, fail.
   if (!curr.is_empty())
     {
       if (curr.type == conv_literal)
-	res.push_back(curr);      
+	res.push_back(curr);
       else
 	throw parse_error("trailing incomplete print format conversion");
     }
@@ -609,20 +715,17 @@ print_format::string_to_components(string const & str)
 
 void print_format::print (ostream& o) const
 {
-  string name = (string(print_to_stream ? "" : "s") 
-		 + string("print") 
-		 + string(print_with_format ? "f" : ""));
-  o << name << "(";
+  o << tok->content << "(";
   if (print_with_format)
-    {
-      o << lex_cast_qstring (raw_components);
-    }
+    o << lex_cast_qstring (raw_components);
+  if (print_with_delim)
+    o << lex_cast_qstring (delimiter.literal_string);
   if (hist)
     hist->print(o);
   for (vector<expression*>::const_iterator i = args.begin();
        i != args.end(); ++i)
     {
-      if (i != args.begin() || print_with_format)
+      if (i != args.begin() || print_with_format || print_with_delim)
 	o << ", ";
       (*i)->print(o);
     }
@@ -649,7 +752,7 @@ void stat_op::print (ostream& o) const
     case sc_min:
       o << "min(";
       break;
-      
+
     case sc_max:
       o << "max(";
       break;
@@ -658,7 +761,7 @@ void stat_op::print (ostream& o) const
   o << ")";
 }
 
-void 
+void
 hist_op::print (ostream& o) const
 {
   o << '@';
@@ -676,13 +779,9 @@ hist_op::print (ostream& o) const
       break;
 
     case hist_log:
-      assert(params.size() == 1);
+      assert(params.size() == 0);
       o << "hist_log(";
       stat->print(o);
-      for (size_t i = 0; i < params.size(); ++i)
-	{
-	  o << ", " << params[i];
-	}
       o << ")";
       break;
     }
@@ -710,6 +809,14 @@ void block::print (ostream& o) const
   o << "}";
 }
 
+block::block (statement* car, statement* cdr)
+{
+  statements.push_back(car);
+  statements.push_back(cdr);
+  this->tok = car->tok;
+}
+
+
 
 void for_loop::print (ostream& o) const
 {
@@ -750,7 +857,7 @@ void foreach_loop::print (ostream& o) const
 
 void null_statement::print (ostream& o) const
 {
-  o << ";"; 
+  o << ";";
 }
 
 
@@ -839,14 +946,28 @@ void probe::print (ostream& o) const
 
 void probe::printsig (ostream& o) const
 {
+  const probe_alias *alias = get_alias ();
+  if (alias)
+    {
+      alias->printsig (o);
+      return;
+    }
+
   for (unsigned i=0; i<locations.size(); i++)
     {
-      if (i > 0) o << "," << endl;
+      if (i > 0) o << ",";
       locations[i]->print (o);
     }
 }
 
 
+void
+probe::collect_derivation_chain (std::vector<probe*> &probes_list)
+{
+  probes_list.push_back(this);
+}
+
+
 void probe_point::print (ostream& o) const
 {
   for (unsigned i=0; i<components.size(); i++)
@@ -857,8 +978,32 @@ void probe_point::print (ostream& o) const
       if (c->arg)
         o << "(" << *c->arg << ")";
     }
-  if (optional)
+  if (sufficient)
+    o << "!";
+  else if (optional) // sufficient implies optional
     o << "?";
+  if (condition)
+    o<< " if (" << *condition << ")";
+}
+
+string probe_point::str ()
+{
+  ostringstream o;
+  for (unsigned i=0; i<components.size(); i++)
+    {
+      if (i>0) o << ".";
+      probe_point::component* c = components[i];
+      o << c->functor;
+      if (c->arg)
+        o << "(" << *c->arg << ")";
+    }
+  if (sufficient)
+    o << "!";
+  else if (optional) // sufficient implies optional
+    o << "?";
+  if (condition)
+    o<< " if (" << *condition << ")";
+  return o.str();
 }
 
 
@@ -877,7 +1022,7 @@ void probe_alias::printsig (ostream& o) const
   o << " = ";
   for (unsigned i=0; i<locations.size(); i++)
     {
-      o << (i>0 ? ", " : "");
+      if (i > 0) o << ", ";
       locations[i]->print (o);
     }
 }
@@ -1068,7 +1213,7 @@ symbol::visit (visitor* u)
   u->visit_symbol (this);
 }
 
-void 
+void
 target_symbol::visit (visitor* u)
 {
   u->visit_target_symbol(this);
@@ -1104,7 +1249,7 @@ hist_op::visit (visitor *u)
   u->visit_hist_op (this);
 }
 
-void 
+void
 indexable::print_indexable (std::ostream& o) const
 {
   const symbol *sym;
@@ -1116,10 +1261,10 @@ indexable::print_indexable (std::ostream& o) const
     {
       assert (hist);
       hist->print (o);
-    }  
+    }
 }
 
-void 
+void
 indexable::visit_indexable (visitor* u)
 {
   symbol *sym;
@@ -1135,42 +1280,42 @@ indexable::visit_indexable (visitor* u)
 }
 
 
-bool 
+bool
 indexable::is_symbol(symbol *& sym_out)
 {
   sym_out = NULL;
   return false;
 }
 
-bool 
+bool
 indexable::is_hist_op(hist_op *& hist_out)
 {
   hist_out = NULL;
   return false;
 }
 
-bool 
+bool
 indexable::is_const_symbol(const symbol *& sym_out) const
 {
   sym_out = NULL;
   return false;
 }
 
-bool 
+bool
 indexable::is_const_hist_op(const hist_op *& hist_out) const
 {
   hist_out = NULL;
   return false;
 }
 
-bool 
+bool
 symbol::is_symbol(symbol *& sym_out)
 {
   sym_out = this;
   return true;
 }
 
-bool 
+bool
 symbol::is_const_symbol(const symbol *& sym_out) const
 {
   sym_out = this;
@@ -1183,14 +1328,14 @@ symbol::get_tok() const
   return tok;
 }
 
-bool 
+bool
 hist_op::is_hist_op(hist_op *& hist_out)
 {
   hist_out = this;
   return true;
 }
 
-bool 
+bool
 hist_op::is_const_hist_op(const hist_op *& hist_out) const
 {
   hist_out = this;
@@ -1206,7 +1351,7 @@ hist_op::get_tok() const
 void
 classify_indexable(indexable* ix,
 		   symbol *& array_out,
-		   hist_op *& hist_out) 
+		   hist_op *& hist_out)
 {
   array_out = NULL;
   hist_out = NULL;
@@ -1219,7 +1364,7 @@ classify_indexable(indexable* ix,
 void
 classify_const_indexable(const indexable* ix,
 			 const symbol *& array_out,
-			 const hist_op *& hist_out) 
+			 const hist_op *& hist_out)
 {
   array_out = NULL;
   hist_out = NULL;
@@ -1229,7 +1374,7 @@ classify_const_indexable(const indexable* ix,
 
 // ------------------------------------------------------------------------
 
-bool 
+bool
 visitor::is_active_lvalue(expression *e)
 {
   for (unsigned i = 0; i < active_lvalues.size(); ++i)
@@ -1240,13 +1385,13 @@ visitor::is_active_lvalue(expression *e)
   return false;
 }
 
-void 
+void
 visitor::push_active_lvalue(expression *e)
 {
   active_lvalues.push_back(e);
 }
 
-void 
+void
 visitor::pop_active_lvalue()
 {
   assert(!active_lvalues.empty());
@@ -1301,7 +1446,7 @@ traversing_visitor::visit_for_loop (for_loop* s)
 void
 traversing_visitor::visit_foreach_loop (foreach_loop* s)
 {
-  symbol *array = NULL;  
+  symbol *array = NULL;
   hist_op *hist = NULL;
   classify_indexable (s->base, array, hist);
   if (array)
@@ -1485,7 +1630,7 @@ traversing_visitor::visit_hist_op (hist_op* e)
 
 
 void
-functioncall_traversing_visitor::visit_functioncall (functioncall* e) 
+functioncall_traversing_visitor::visit_functioncall (functioncall* e)
 {
   traversing_visitor::visit_functioncall (e);
 
@@ -1514,7 +1659,7 @@ varuse_collecting_visitor::visit_embeddedcode (embeddedcode *s)
   // Kludge(tm): we look for a magic string within the function body.
   // $target variables as rvalues will have this; lvalues won't.
   // Also, explicit side-effect-free tapset functions will have this.
-  
+
   assert (current_function); // only they get embedded code
   if (s->code.find ("/* pure */") != string::npos)
     return;
@@ -1522,6 +1667,17 @@ varuse_collecting_visitor::visit_embeddedcode (embeddedcode *s)
   embedded_seen = true;
 }
 
+void
+varuse_collecting_visitor::visit_target_symbol (target_symbol *e)
+{
+  // Still-unresolved target symbol assignments get treated as
+  // generating side-effects like embedded-C, to prevent premature
+  // elision and later error message suppression (PR5516).  rvalue use
+  // of unresolved target symbols is OTOH not considered a side-effect.
+
+  if (is_active_lvalue (e))
+    embedded_seen = true;
+}
 
 void
 varuse_collecting_visitor::visit_print_format (print_format* e)
@@ -1529,7 +1685,7 @@ varuse_collecting_visitor::visit_print_format (print_format* e)
   // NB: Instead of being top-level statements, "print" and "printf"
   // are implemented as statement-expressions containing a
   // print_format.  They have side-effects, but not via the
-  // embedded-code detection method above. 
+  // embedded-code detection method above.
   //
   // But sprint and sprintf don't have side-effects.
 
@@ -1565,6 +1721,15 @@ varuse_collecting_visitor::visit_symbol (symbol *e)
   if (e->referent == 0)
     throw semantic_error ("symbol without referent", e->tok);
 
+  // We could handle initialized globals by marking them as "written".
+  // However, this current visitor may be called for a function or
+  // probe body, from the point of view of which this global is
+  // already initialized, so not written.
+  /*
+  if (e->referent->init)
+    written.insert (e->referent);
+  */
+
   if (current_lvalue == e || current_lrvalue == e)
     {
       written.insert (e->referent);
@@ -1641,18 +1806,41 @@ varuse_collecting_visitor::visit_post_crement (post_crement *e)
 void
 varuse_collecting_visitor::visit_foreach_loop (foreach_loop* s)
 {
-  functioncall_traversing_visitor::visit_foreach_loop (s);
+  // NB: we duplicate so don't bother call
+  // functioncall_traversing_visitor::visit_foreach_loop (s);
+
+  symbol *array = NULL;
+  hist_op *hist = NULL;
+  classify_indexable (s->base, array, hist);
+  if (array)
+    array->visit(this);
+  else
+    hist->visit(this);
+
   // If the collection is sorted, imply a "write" access to the
-  // array in addition to the "read" one already noted in the
-  // base class call above.
+  // array in addition to the "read" one already noted above.
   if (s->sort_direction)
     {
-      symbol *array = NULL;  
+      symbol *array = NULL;
       hist_op *hist = NULL;
       classify_indexable (s->base, array, hist);
       if (array) this->written.insert (array->referent);
       // XXX: Can hist_op iterations be sorted?
     }
+
+  // NB: don't forget to visit the index expressions, which are lvalues.
+  for (unsigned i=0; i<s->indexes.size(); i++)
+    {
+      expression* last_lvalue = current_lvalue;
+      current_lvalue = s->indexes[i]; // leave a mark for ::visit_symbol
+      s->indexes[i]->visit (this);
+      current_lvalue = last_lvalue;
+    }
+
+  if (s->limit)
+    s->limit->visit (this);
+
+  s->block->visit (this);
 }
 
 
@@ -1909,374 +2097,466 @@ throwing_visitor::visit_hist_op (hist_op* e)
 // ------------------------------------------------------------------------
 
 
+void
+update_visitor::visit_block (block* s)
+{
+  for (unsigned i = 0; i < s->statements.size(); ++i)
+    s->statements[i] = require (s->statements[i]);
+  provide (s);
+}
+
+void
+update_visitor::visit_embeddedcode (embeddedcode* s)
+{
+  provide (s);
+}
+
+void
+update_visitor::visit_null_statement (null_statement* s)
+{
+  provide (s);
+}
+
+void
+update_visitor::visit_expr_statement (expr_statement* s)
+{
+  s->value = require (s->value);
+  provide (s);
+}
+
+void
+update_visitor::visit_if_statement (if_statement* s)
+{
+  s->condition = require (s->condition);
+  s->thenblock = require (s->thenblock);
+  s->elseblock = require (s->elseblock);
+  provide (s);
+}
+
+void
+update_visitor::visit_for_loop (for_loop* s)
+{
+  s->init = require (s->init);
+  s->cond = require (s->cond);
+  s->incr = require (s->incr);
+  s->block = require (s->block);
+  provide (s);
+}
+
+void
+update_visitor::visit_foreach_loop (foreach_loop* s)
+{
+  for (unsigned i = 0; i < s->indexes.size(); ++i)
+    s->indexes[i] = require (s->indexes[i]);
+  s->base = require (s->base);
+  s->limit = require (s->limit);
+  s->block = require (s->block);
+  provide (s);
+}
+
+void
+update_visitor::visit_return_statement (return_statement* s)
+{
+  s->value = require (s->value);
+  provide (s);
+}
+
+void
+update_visitor::visit_delete_statement (delete_statement* s)
+{
+  s->value = require (s->value);
+  provide (s);
+}
+
+void
+update_visitor::visit_next_statement (next_statement* s)
+{
+  provide (s);
+}
+
+void
+update_visitor::visit_break_statement (break_statement* s)
+{
+  provide (s);
+}
+
+void
+update_visitor::visit_continue_statement (continue_statement* s)
+{
+  provide (s);
+}
+
+void
+update_visitor::visit_literal_string (literal_string* e)
+{
+  provide (e);
+}
+
+void
+update_visitor::visit_literal_number (literal_number* e)
+{
+  provide (e);
+}
+
+void
+update_visitor::visit_binary_expression (binary_expression* e)
+{
+  e->left = require (e->left);
+  e->right = require (e->right);
+  provide (e);
+}
+
+void
+update_visitor::visit_unary_expression (unary_expression* e)
+{
+  e->operand = require (e->operand);
+  provide (e);
+}
+
+void
+update_visitor::visit_pre_crement (pre_crement* e)
+{
+  e->operand = require (e->operand);
+  provide (e);
+}
+
+void
+update_visitor::visit_post_crement (post_crement* e)
+{
+  e->operand = require (e->operand);
+  provide (e);
+}
+
+
+void
+update_visitor::visit_logical_or_expr (logical_or_expr* e)
+{
+  e->left = require (e->left);
+  e->right = require (e->right);
+  provide (e);
+}
+
+void
+update_visitor::visit_logical_and_expr (logical_and_expr* e)
+{
+  e->left = require (e->left);
+  e->right = require (e->right);
+  provide (e);
+}
+
+void
+update_visitor::visit_array_in (array_in* e)
+{
+  e->operand = require (e->operand);
+  provide (e);
+}
+
+void
+update_visitor::visit_comparison (comparison* e)
+{
+  e->left = require (e->left);
+  e->right = require (e->right);
+  provide (e);
+}
+
+void
+update_visitor::visit_concatenation (concatenation* e)
+{
+  e->left = require (e->left);
+  e->right = require (e->right);
+  provide (e);
+}
+
+void
+update_visitor::visit_ternary_expression (ternary_expression* e)
+{
+  e->cond = require (e->cond);
+  e->truevalue = require (e->truevalue);
+  e->falsevalue = require (e->falsevalue);
+  provide (e);
+}
+
+void
+update_visitor::visit_assignment (assignment* e)
+{
+  e->left = require (e->left);
+  e->right = require (e->right);
+  provide (e);
+}
+
+void
+update_visitor::visit_symbol (symbol* e)
+{
+  provide (e);
+}
+
+void
+update_visitor::visit_target_symbol (target_symbol* e)
+{
+  provide (e);
+}
+
+void
+update_visitor::visit_arrayindex (arrayindex* e)
+{
+  e->base = require (e->base);
+  for (unsigned i = 0; i < e->indexes.size(); ++i)
+    e->indexes[i] = require (e->indexes[i]);
+  provide (e);
+}
+
+void
+update_visitor::visit_functioncall (functioncall* e)
+{
+  for (unsigned i = 0; i < e->args.size(); ++i)
+    e->args[i] = require (e->args[i]);
+  provide (e);
+}
+
+void
+update_visitor::visit_print_format (print_format* e)
+{
+  for (unsigned i = 0; i < e->args.size(); ++i)
+    e->args[i] = require (e->args[i]);
+  e->hist = require (e->hist);
+  provide (e);
+}
+
+void
+update_visitor::visit_stat_op (stat_op* e)
+{
+  e->stat = require (e->stat);
+  provide (e);
+}
+
+void
+update_visitor::visit_hist_op (hist_op* e)
+{
+  e->stat = require (e->stat);
+  provide (e);
+}
+
+template <> indexable*
+update_visitor::require <indexable*> (indexable* src, bool clearok)
+{
+  indexable *dst = NULL;
+  if (src != NULL)
+    {
+      symbol *array_src=NULL;
+      hist_op *hist_src=NULL;
+
+      classify_indexable(src, array_src, hist_src);
+
+      if (array_src)
+        dst = require (array_src);
+      else
+        dst = require (hist_src);
+      assert(clearok || dst);
+    }
+  return dst;
+}
+
+
+// ------------------------------------------------------------------------
+
+
 void
 deep_copy_visitor::visit_block (block* s)
 {
-  block* n = new block;
-  n->tok = s->tok;
-  for (unsigned i = 0; i < s->statements.size(); ++i)
-    {
-      statement* ns;
-      require <statement*> (this, &ns, s->statements[i]);
-      n->statements.push_back(ns);
-    }
-  provide <block*> (this, n);
+  update_visitor::visit_block(new block(*s));
 }
 
 void
 deep_copy_visitor::visit_embeddedcode (embeddedcode* s)
 {
-  embeddedcode* n = new embeddedcode;
-  n->tok = s->tok;
-  n->code = s->code;
-  provide <embeddedcode*> (this, n);
+  update_visitor::visit_embeddedcode(new embeddedcode(*s));
 }
 
 void
 deep_copy_visitor::visit_null_statement (null_statement* s)
 {
-  null_statement* n = new null_statement;
-  n->tok = s->tok;
-  provide <null_statement*> (this, n);
+  update_visitor::visit_null_statement(new null_statement(*s));
 }
 
 void
 deep_copy_visitor::visit_expr_statement (expr_statement* s)
 {
-  expr_statement* n = new expr_statement;
-  n->tok = s->tok;
-  require <expression*> (this, &(n->value), s->value);
-  provide <expr_statement*> (this, n);
+  update_visitor::visit_expr_statement(new expr_statement(*s));
 }
 
 void
 deep_copy_visitor::visit_if_statement (if_statement* s)
 {
-  if_statement* n = new if_statement;
-  n->tok = s->tok;
-  require <expression*> (this, &(n->condition), s->condition);
-  require <statement*> (this, &(n->thenblock), s->thenblock);
-  require <statement*> (this, &(n->elseblock), s->elseblock);
-  provide <if_statement*> (this, n);
+  update_visitor::visit_if_statement(new if_statement(*s));
 }
 
 void
 deep_copy_visitor::visit_for_loop (for_loop* s)
 {
-  for_loop* n = new for_loop;
-  n->tok = s->tok;
-  require <expr_statement*> (this, &(n->init), s->init);
-  require <expression*> (this, &(n->cond), s->cond);
-  require <expr_statement*> (this, &(n->incr), s->incr);
-  require <statement*> (this, &(n->block), s->block);  
-  provide <for_loop*> (this, n);
+  update_visitor::visit_for_loop(new for_loop(*s));
 }
 
 void
 deep_copy_visitor::visit_foreach_loop (foreach_loop* s)
 {
-  foreach_loop* n = new foreach_loop;
-  n->tok = s->tok;
-  for (unsigned i = 0; i < s->indexes.size(); ++i)
-    {
-      symbol* sym;
-      require <symbol*> (this, &sym, s->indexes[i]);
-      n->indexes.push_back(sym);
-    }
-
-  require <indexable*> (this, &(n->base), s->base);
-
-  n->sort_direction = s->sort_direction;
-  n->sort_column = s->sort_column;
-  require <expression*> (this, &(n->limit), s->limit);
-
-  require <statement*> (this, &(n->block), s->block);
-  provide <foreach_loop*> (this, n);
+  update_visitor::visit_foreach_loop(new foreach_loop(*s));
 }
 
 void
 deep_copy_visitor::visit_return_statement (return_statement* s)
 {
-  return_statement* n = new return_statement;
-  n->tok = s->tok;
-  require <expression*> (this, &(n->value), s->value);
-  provide <return_statement*> (this, n);
+  update_visitor::visit_return_statement(new return_statement(*s));
 }
 
 void
 deep_copy_visitor::visit_delete_statement (delete_statement* s)
 {
-  delete_statement* n = new delete_statement;
-  n->tok = s->tok;
-  require <expression*> (this, &(n->value), s->value);
-  provide <delete_statement*> (this, n);
+  update_visitor::visit_delete_statement(new delete_statement(*s));
 }
 
 void
 deep_copy_visitor::visit_next_statement (next_statement* s)
 {
-  next_statement* n = new next_statement;
-  n->tok = s->tok;
-  provide <next_statement*> (this, n);
+  update_visitor::visit_next_statement(new next_statement(*s));
 }
 
 void
 deep_copy_visitor::visit_break_statement (break_statement* s)
 {
-  break_statement* n = new break_statement;
-  n->tok = s->tok;
-  provide <break_statement*> (this, n);
+  update_visitor::visit_break_statement(new break_statement(*s));
 }
 
 void
 deep_copy_visitor::visit_continue_statement (continue_statement* s)
 {
-  continue_statement* n = new continue_statement;
-  n->tok = s->tok;
-  provide <continue_statement*> (this, n);
+  update_visitor::visit_continue_statement(new continue_statement(*s));
 }
 
 void
 deep_copy_visitor::visit_literal_string (literal_string* e)
 {
-  literal_string* n = new literal_string(e->value);
-  n->tok = e->tok;
-  provide <literal_string*> (this, n);
+  update_visitor::visit_literal_string(new literal_string(*e));
 }
 
 void
 deep_copy_visitor::visit_literal_number (literal_number* e)
 {
-  literal_number* n = new literal_number(e->value);
-  n->tok = e->tok;
-  provide <literal_number*> (this, n);
+  update_visitor::visit_literal_number(new literal_number(*e));
 }
 
 void
 deep_copy_visitor::visit_binary_expression (binary_expression* e)
 {
-  binary_expression* n = new binary_expression;
-  n->op = e->op;
-  n->tok = e->tok;
-  require <expression*> (this, &(n->left), e->left);
-  require <expression*> (this, &(n->right), e->right);
-  provide <binary_expression*> (this, n);
+  update_visitor::visit_binary_expression(new binary_expression(*e));
 }
 
 void
 deep_copy_visitor::visit_unary_expression (unary_expression* e)
 {
-  unary_expression* n = new unary_expression;
-  n->op = e->op;
-  n->tok = e->tok;
-  require <expression*> (this, &(n->operand), e->operand);
-  provide <unary_expression*> (this, n);
+  update_visitor::visit_unary_expression(new unary_expression(*e));
 }
 
 void
 deep_copy_visitor::visit_pre_crement (pre_crement* e)
 {
-  pre_crement* n = new pre_crement;
-  n->op = e->op;
-  n->tok = e->tok;
-  require <expression*> (this, &(n->operand), e->operand);
-  provide <pre_crement*> (this, n);
+  update_visitor::visit_pre_crement(new pre_crement(*e));
 }
 
 void
 deep_copy_visitor::visit_post_crement (post_crement* e)
 {
-  post_crement* n = new post_crement;
-  n->op = e->op;
-  n->tok = e->tok;
-  require <expression*> (this, &(n->operand), e->operand);
-  provide <post_crement*> (this, n);
+  update_visitor::visit_post_crement(new post_crement(*e));
 }
 
 
 void
 deep_copy_visitor::visit_logical_or_expr (logical_or_expr* e)
 {
-  logical_or_expr* n = new logical_or_expr;
-  n->op = e->op;
-  n->tok = e->tok;
-  require <expression*> (this, &(n->left), e->left);
-  require <expression*> (this, &(n->right), e->right);
-  provide <logical_or_expr*> (this, n);
+  update_visitor::visit_logical_or_expr(new logical_or_expr(*e));
 }
 
 void
 deep_copy_visitor::visit_logical_and_expr (logical_and_expr* e)
 {
-  logical_and_expr* n = new logical_and_expr;
-  n->op = e->op;
-  n->tok = e->tok;
-  require <expression*> (this, &(n->left), e->left);
-  require <expression*> (this, &(n->right), e->right);
-  provide <logical_and_expr*> (this, n);
+  update_visitor::visit_logical_and_expr(new logical_and_expr(*e));
 }
 
 void
 deep_copy_visitor::visit_array_in (array_in* e)
 {
-  array_in* n = new array_in;
-  n->tok = e->tok;
-  require <arrayindex*> (this, &(n->operand), e->operand);
-  provide <array_in*> (this, n);
+  update_visitor::visit_array_in(new array_in(*e));
 }
 
 void
 deep_copy_visitor::visit_comparison (comparison* e)
 {
-  comparison* n = new comparison;
-  n->op = e->op;
-  n->tok = e->tok;
-  require <expression*> (this, &(n->left), e->left);
-  require <expression*> (this, &(n->right), e->right);
-  provide <comparison*> (this, n);
+  update_visitor::visit_comparison(new comparison(*e));
 }
 
 void
 deep_copy_visitor::visit_concatenation (concatenation* e)
 {
-  concatenation* n = new concatenation;
-  n->op = e->op;
-  n->tok = e->tok;
-  require <expression*> (this, &(n->left), e->left);
-  require <expression*> (this, &(n->right), e->right);
-  provide <concatenation*> (this, n);
+  update_visitor::visit_concatenation(new concatenation(*e));
 }
 
 void
 deep_copy_visitor::visit_ternary_expression (ternary_expression* e)
 {
-  ternary_expression* n = new ternary_expression;
-  n->tok = e->tok;
-  require <expression*> (this, &(n->cond), e->cond);
-  require <expression*> (this, &(n->truevalue), e->truevalue);
-  require <expression*> (this, &(n->falsevalue), e->falsevalue);
-  provide <ternary_expression*> (this, n);
+  update_visitor::visit_ternary_expression(new ternary_expression(*e));
 }
 
 void
 deep_copy_visitor::visit_assignment (assignment* e)
 {
-  assignment* n = new assignment;
-  n->op = e->op;
-  n->tok = e->tok;
-  require <expression*> (this, &(n->left), e->left);
-  require <expression*> (this, &(n->right), e->right);
-  provide <assignment*> (this, n);
+  update_visitor::visit_assignment(new assignment(*e));
 }
 
 void
 deep_copy_visitor::visit_symbol (symbol* e)
 {
-  symbol* n = new symbol;
-  n->tok = e->tok;
-  n->name = e->name;
-  n->referent = NULL;
-  provide <symbol*> (this, n);
+  symbol* n = new symbol(*e);
+  n->referent = NULL; // don't copy!
+  update_visitor::visit_symbol(n);
 }
 
 void
 deep_copy_visitor::visit_target_symbol (target_symbol* e)
 {
-  target_symbol* n = new target_symbol;
-  n->tok = e->tok;
-  n->base_name = e->base_name;
-  n->components = e->components;
-  provide <target_symbol*> (this, n);
+  target_symbol* n = new target_symbol(*e);
+  n->referent = NULL; // don't copy!
+  update_visitor::visit_target_symbol(n);
 }
 
 void
 deep_copy_visitor::visit_arrayindex (arrayindex* e)
 {
-  arrayindex* n = new arrayindex;
-  n->tok = e->tok;
-
-  require <indexable*> (this, &(n->base), e->base);
-
-  for (unsigned i = 0; i < e->indexes.size(); ++i)
-    {
-      expression* ne;
-      require <expression*> (this, &ne, e->indexes[i]);
-      n->indexes.push_back(ne);
-    }
-  provide <arrayindex*> (this, n);
+  update_visitor::visit_arrayindex(new arrayindex(*e));
 }
 
 void
 deep_copy_visitor::visit_functioncall (functioncall* e)
 {
-  functioncall* n = new functioncall;
-  n->tok = e->tok;
-  n->function = e->function;
-  n->referent = NULL;
-  for (unsigned i = 0; i < e->args.size(); ++i)
-    {
-      expression* na;
-      require <expression*> (this, &na, e->args[i]);
-      n->args.push_back(na);
-    }
-  provide <functioncall*> (this, n);
+  functioncall* n = new functioncall(*e);
+  n->referent = NULL; // don't copy!
+  update_visitor::visit_functioncall(n);
 }
 
 void
 deep_copy_visitor::visit_print_format (print_format* e)
 {
-  print_format* n = new print_format;
-  n->tok = e->tok;
-  n->print_with_format = e->print_with_format;
-  n->print_to_stream = e->print_to_stream;
-  n->raw_components = e->raw_components;
-  n->components = e->components;
-  for (unsigned i = 0; i < e->args.size(); ++i)
-    {
-      expression* na;
-      require <expression*> (this, &na, e->args[i]);
-      n->args.push_back(na);
-    }
-  if (e->hist)
-    require <hist_op*> (this, &n->hist, e->hist);
-  provide <print_format*> (this, n);
+  update_visitor::visit_print_format(new print_format(*e));
 }
 
 void
 deep_copy_visitor::visit_stat_op (stat_op* e)
 {
-  stat_op* n = new stat_op;
-  n->tok = e->tok;
-  n->ctype = e->ctype;
-  require <expression*> (this, &(n->stat), e->stat);
-  provide <stat_op*> (this, n);
+  update_visitor::visit_stat_op(new stat_op(*e));
 }
 
 void
 deep_copy_visitor::visit_hist_op (hist_op* e)
 {
-  hist_op* n = new hist_op;
-  n->tok = e->tok;
-  n->htype = e->htype;
-  n->params = e->params;
-  require <expression*> (this, &(n->stat), e->stat);
-  provide <hist_op*> (this, n);
+  update_visitor::visit_hist_op(new hist_op(*e));
 }
 
-block* 
-deep_copy_visitor::deep_copy (block* b)
-{
-  block* n;
-  deep_copy_visitor v;
-  require <block*> (&v, &n, b);
-  return n;
-}
-
-statement* 
-deep_copy_visitor::deep_copy (statement* s)
-{
-  statement* n;
-  deep_copy_visitor v;
-  require <statement*> (&v, &n, s);
-  return n;
-}
+/* vim: set sw=2 ts=8 cino=>4,n-2,{2,^-2,t0,(0,u0,w1,M1 : */
