@@ -13,6 +13,12 @@
 #include "staptree.h"
 #include "elaborate.h"
 
-void register_standard_tapsets(systemtap_session & sess);
+struct derived_probe_group;
+
+void register_standard_tapsets(systemtap_session& sess);
+std::vector<derived_probe_group*> all_session_groups(systemtap_session& s);
+
 
 #endif // TAPSETS_H
+
+/* vim: set sw=2 ts=8 cino=>4,n-2,{2,^-2,t0,(0,u0,w1,M1 : */
